# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ft_sessions.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ft_error_pb2 as ft__error__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
import ft_types_pb2 as ft__types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x11\x66t_sessions.proto\x12\x07\x66oxtrot\x1a\x0e\x66t_error.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x0e\x66t_types.proto\"\xb8\x01\n\x0csession_info\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\x11\n\tsessionid\x18\x02 \x01(\x0c\x12\x17\n\x0fuser_identifier\x18\x03 \x01(\t\x12\x0f\n\x07\x63omment\x18\x04 \x01(\t\x12\x0f\n\x07\x64\x65vices\x18\x05 \x03(\r\x12\r\n\x05\x66lags\x18\x06 \x03(\t\x12*\n\x06\x65xpiry\x18\x07 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"X\n\x0csession_list\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\'\n\x08sessions\x18\x02 \x03(\x0b\x32\x15.foxtrot.session_info2\xff\x01\n\x08sessions\x12<\n\x0cStartSession\x12\x15.foxtrot.session_info\x1a\x15.foxtrot.session_info\x12<\n\x0c\x43loseSession\x12\x15.foxtrot.session_info\x1a\x15.foxtrot.session_info\x12@\n\x10KeepAliveSession\x12\x15.foxtrot.session_info\x1a\x15.foxtrot.session_info\x12\x35\n\x0cListSessions\x12\x0e.foxtrot.empty\x1a\x15.foxtrot.session_listb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ft_sessions_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_SESSION_INFO']._serialized_start=96
  _globals['_SESSION_INFO']._serialized_end=280
  _globals['_SESSION_LIST']._serialized_start=282
  _globals['_SESSION_LIST']._serialized_end=370
  _globals['_SESSIONS']._serialized_start=373
  _globals['_SESSIONS']._serialized_end=628
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Apr 19 17:47:55 2017

@author: weatherill
"""

from .foxtrot_pb2 import telemetry
import nanomsg

from datetime import datetime, timedelta

EPOCH = datetime.fromtimestamp(0)

def dt_from_u64(val):
    since_epoch = timedelta(0,0,val)
    out = EPOCH + since_epoch
    return out

class TelemetryCatcher:
    def __init__(self,connstr, topic):
#        if not isinstance(connstr, bytes):
#            connstr = connstr.encode("ASCII")
#            
        self._sock = nanomsg.Socket(nanomsg.SUB)
        self._sock.connect(connstr)
#        if not isinstance(topic,bytes):
#            topic = topic.encode("ASCII")
        self._topic = topic
        self._subscriptions = {}
        
        
    def subscribe(self,subtopic, callback):
#        if not isinstance(subtopic,bytes):
#            subtopic = subtopic.encode("ASCII")
#            
#        substr = self._topic + b'|' + subtopic
        substr = self._topic + '|' + subtopic
        print(substr)
        self._sock.set_string_option(nanomsg.SUB, 
                                     nanomsg.SUB_SUBSCRIBE,
                                     substr)
        
        self._subscriptions[substr] = callback
    
    def waitmessage(self):
        msg = self._sock.recv()
        splitpoint = msg.index(b'>')
        
        topic = msg[:splitpoint]
        protobuf = msg[splitpoint+1:]
        print(topic)
        
        t = telemetry()
        t.ParseFromString(protobuf)        
        retattr = t.WhichOneof('return')
        return (topic, dt_from_u64(t.tstamp), getattr(t,retattr))
    

    def process_telems_forever(self):
        while True:
            topic, tstamp, ret = self.waitmessage()
            key = topic.decode("ASCII")
            if key in self._subscriptions:
                fun = self._subscriptions[key]
                print("calling function...")
                fun(topic,tstamp,ret)
            
            
    
    def __del__(self):
        self._sock.close()
        
        


import ft_error_pb2 as _ft_error_pb2
import ft_types_pb2 as _ft_types_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class serverflag(_message.Message):
    __slots__ = ["err", "msgid", "flagname", "dblval", "intval", "boolval", "stringval"]
    ERR_FIELD_NUMBER: _ClassVar[int]
    MSGID_FIELD_NUMBER: _ClassVar[int]
    FLAGNAME_FIELD_NUMBER: _ClassVar[int]
    DBLVAL_FIELD_NUMBER: _ClassVar[int]
    INTVAL_FIELD_NUMBER: _ClassVar[int]
    BOOLVAL_FIELD_NUMBER: _ClassVar[int]
    STRINGVAL_FIELD_NUMBER: _ClassVar[int]
    err: _ft_error_pb2.errstatus
    msgid: int
    flagname: str
    dblval: float
    intval: int
    boolval: bool
    stringval: str
    def __init__(self, err: _Optional[_Union[_ft_error_pb2.errstatus, _Mapping]] = ..., msgid: _Optional[int] = ..., flagname: _Optional[str] = ..., dblval: _Optional[float] = ..., intval: _Optional[int] = ..., boolval: bool = ..., stringval: _Optional[str] = ...) -> None: ...

class serverflaglist(_message.Message):
    __slots__ = ["err", "msgid", "flags"]
    ERR_FIELD_NUMBER: _ClassVar[int]
    MSGID_FIELD_NUMBER: _ClassVar[int]
    FLAGS_FIELD_NUMBER: _ClassVar[int]
    err: _ft_error_pb2.errstatus
    msgid: int
    flags: _containers.RepeatedCompositeFieldContainer[serverflag]
    def __init__(self, err: _Optional[_Union[_ft_error_pb2.errstatus, _Mapping]] = ..., msgid: _Optional[int] = ..., flags: _Optional[_Iterable[_Union[serverflag, _Mapping]]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ft_flags.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ft_error_pb2 as ft__error__pb2
import ft_types_pb2 as ft__types__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x66t_flags.proto\x12\x07\x66oxtrot\x1a\x0e\x66t_error.proto\x1a\x0e\x66t_types.proto\"\xa1\x01\n\nserverflag\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\r\n\x05msgid\x18\x02 \x01(\r\x12\x10\n\x08\x66lagname\x18\x03 \x01(\t\x12\x10\n\x06\x64\x62lval\x18\x04 \x01(\x01H\x00\x12\x10\n\x06intval\x18\x05 \x01(\x11H\x00\x12\x11\n\x07\x62oolval\x18\x06 \x01(\x08H\x00\x12\x13\n\tstringval\x18\x07 \x01(\tH\x00\x42\x05\n\x03\x61rg\"d\n\x0eserverflaglist\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\r\n\x05msgid\x18\x02 \x01(\r\x12\"\n\x05\x66lags\x18\x03 \x03(\x0b\x32\x13.foxtrot.serverflag2\xf5\x01\n\x05\x66lags\x12\x39\n\rSetServerFlag\x12\x13.foxtrot.serverflag\x1a\x13.foxtrot.serverflag\x12\x39\n\rGetServerFlag\x12\x13.foxtrot.serverflag\x1a\x13.foxtrot.serverflag\x12:\n\x0fListServerFlags\x12\x0e.foxtrot.empty\x1a\x17.foxtrot.serverflaglist\x12:\n\x0e\x44ropServerFlag\x12\x13.foxtrot.serverflag\x1a\x13.foxtrot.serverflagB\x0c\n\nio.foxtrotb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ft_flags_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\nio.foxtrot'
  _globals['_SERVERFLAG']._serialized_start=60
  _globals['_SERVERFLAG']._serialized_end=221
  _globals['_SERVERFLAGLIST']._serialized_start=223
  _globals['_SERVERFLAGLIST']._serialized_end=323
  _globals['_FLAGS']._serialized_start=326
  _globals['_FLAGS']._serialized_end=571
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import ft_flags_pb2 as ft__flags__pb2
import ft_types_pb2 as ft__types__pb2


class flagsStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.SetServerFlag = channel.unary_unary(
                '/foxtrot.flags/SetServerFlag',
                request_serializer=ft__flags__pb2.serverflag.SerializeToString,
                response_deserializer=ft__flags__pb2.serverflag.FromString,
                )
        self.GetServerFlag = channel.unary_unary(
                '/foxtrot.flags/GetServerFlag',
                request_serializer=ft__flags__pb2.serverflag.SerializeToString,
                response_deserializer=ft__flags__pb2.serverflag.FromString,
                )
        self.ListServerFlags = channel.unary_unary(
                '/foxtrot.flags/ListServerFlags',
                request_serializer=ft__types__pb2.empty.SerializeToString,
                response_deserializer=ft__flags__pb2.serverflaglist.FromString,
                )
        self.DropServerFlag = channel.unary_unary(
                '/foxtrot.flags/DropServerFlag',
                request_serializer=ft__flags__pb2.serverflag.SerializeToString,
                response_deserializer=ft__flags__pb2.serverflag.FromString,
                )


class flagsServicer(object):
    """Missing associated documentation comment in .proto file."""

    def SetServerFlag(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetServerFlag(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListServerFlags(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DropServerFlag(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_flagsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'SetServerFlag': grpc.unary_unary_rpc_method_handler(
                    servicer.SetServerFlag,
                    request_deserializer=ft__flags__pb2.serverflag.FromString,
                    response_serializer=ft__flags__pb2.serverflag.SerializeToString,
            ),
            'GetServerFlag': grpc.unary_unary_rpc_method_handler(
                    servicer.GetServerFlag,
                    request_deserializer=ft__flags__pb2.serverflag.FromString,
                    response_serializer=ft__flags__pb2.serverflag.SerializeToString,
            ),
            'ListServerFlags': grpc.unary_unary_rpc_method_handler(
                    servicer.ListServerFlags,
                    request_deserializer=ft__types__pb2.empty.FromString,
                    response_serializer=ft__flags__pb2.serverflaglist.SerializeToString,
            ),
            'DropServerFlag': grpc.unary_unary_rpc_method_handler(
                    servicer.DropServerFlag,
                    request_deserializer=ft__flags__pb2.serverflag.FromString,
                    response_serializer=ft__flags__pb2.serverflag.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'foxtrot.flags', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class flags(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def SetServerFlag(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.flags/SetServerFlag',
            ft__flags__pb2.serverflag.SerializeToString,
            ft__flags__pb2.serverflag.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetServerFlag(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.flags/GetServerFlag',
            ft__flags__pb2.serverflag.SerializeToString,
            ft__flags__pb2.serverflag.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListServerFlags(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.flags/ListServerFlags',
            ft__types__pb2.empty.SerializeToString,
            ft__flags__pb2.serverflaglist.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DropServerFlag(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.flags/DropServerFlag',
            ft__flags__pb2.serverflag.SerializeToString,
            ft__flags__pb2.serverflag.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import ft_auth_pb2 as ft__auth__pb2
import ft_types_pb2 as ft__types__pb2


class authStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.RequestAuthChallenge = channel.unary_unary(
                '/foxtrot.auth/RequestAuthChallenge',
                request_serializer=ft__auth__pb2.auth_request.SerializeToString,
                response_deserializer=ft__auth__pb2.auth_challenge.FromString,
                )
        self.RespondAuthChallenge = channel.unary_unary(
                '/foxtrot.auth/RespondAuthChallenge',
                request_serializer=ft__auth__pb2.auth_response.SerializeToString,
                response_deserializer=ft__auth__pb2.auth_confirm.FromString,
                )
        self.GetSupportedAuthMechanisms = channel.unary_unary(
                '/foxtrot.auth/GetSupportedAuthMechanisms',
                request_serializer=ft__types__pb2.empty.SerializeToString,
                response_deserializer=ft__auth__pb2.auth_type_list.FromString,
                )
        self.SaslAuthProcess = channel.stream_stream(
                '/foxtrot.auth/SaslAuthProcess',
                request_serializer=ft__auth__pb2.sasl_auth_data.SerializeToString,
                response_deserializer=ft__auth__pb2.sasl_auth_data.FromString,
                )
        self.VerifyToken = channel.unary_unary(
                '/foxtrot.auth/VerifyToken',
                request_serializer=ft__auth__pb2.sasl_auth_data.SerializeToString,
                response_deserializer=ft__auth__pb2.sasl_auth_data.FromString,
                )


class authServicer(object):
    """Missing associated documentation comment in .proto file."""

    def RequestAuthChallenge(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RespondAuthChallenge(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def GetSupportedAuthMechanisms(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SaslAuthProcess(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def VerifyToken(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_authServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'RequestAuthChallenge': grpc.unary_unary_rpc_method_handler(
                    servicer.RequestAuthChallenge,
                    request_deserializer=ft__auth__pb2.auth_request.FromString,
                    response_serializer=ft__auth__pb2.auth_challenge.SerializeToString,
            ),
            'RespondAuthChallenge': grpc.unary_unary_rpc_method_handler(
                    servicer.RespondAuthChallenge,
                    request_deserializer=ft__auth__pb2.auth_response.FromString,
                    response_serializer=ft__auth__pb2.auth_confirm.SerializeToString,
            ),
            'GetSupportedAuthMechanisms': grpc.unary_unary_rpc_method_handler(
                    servicer.GetSupportedAuthMechanisms,
                    request_deserializer=ft__types__pb2.empty.FromString,
                    response_serializer=ft__auth__pb2.auth_type_list.SerializeToString,
            ),
            'SaslAuthProcess': grpc.stream_stream_rpc_method_handler(
                    servicer.SaslAuthProcess,
                    request_deserializer=ft__auth__pb2.sasl_auth_data.FromString,
                    response_serializer=ft__auth__pb2.sasl_auth_data.SerializeToString,
            ),
            'VerifyToken': grpc.unary_unary_rpc_method_handler(
                    servicer.VerifyToken,
                    request_deserializer=ft__auth__pb2.sasl_auth_data.FromString,
                    response_serializer=ft__auth__pb2.sasl_auth_data.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'foxtrot.auth', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class auth(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def RequestAuthChallenge(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.auth/RequestAuthChallenge',
            ft__auth__pb2.auth_request.SerializeToString,
            ft__auth__pb2.auth_challenge.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RespondAuthChallenge(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.auth/RespondAuthChallenge',
            ft__auth__pb2.auth_response.SerializeToString,
            ft__auth__pb2.auth_confirm.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def GetSupportedAuthMechanisms(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.auth/GetSupportedAuthMechanisms',
            ft__types__pb2.empty.SerializeToString,
            ft__auth__pb2.auth_type_list.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SaslAuthProcess(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/foxtrot.auth/SaslAuthProcess',
            ft__auth__pb2.sasl_auth_data.SerializeToString,
            ft__auth__pb2.sasl_auth_data.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def VerifyToken(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.auth/VerifyToken',
            ft__auth__pb2.sasl_auth_data.SerializeToString,
            ft__auth__pb2.sasl_auth_data.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: foxtrot.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ft_error_pb2 as ft__error__pb2
import ft_types_pb2 as ft__types__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rfoxtrot.proto\x12\x07\x66oxtrot\x1a\x0e\x66t_error.proto\x1a\x0e\x66t_types.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"\xb5\x01\n\x16\x62roadcast_notification\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\r\n\x05msgid\x18\x02 \x01(\r\x12\x19\n\x11use_default_title\x18\x03 \x01(\x08\x12\r\n\x05title\x18\x04 \x01(\t\x12\x0c\n\x04\x62ody\x18\x05 \x01(\t\x12\x1b\n\x13use_default_channel\x18\x06 \x01(\x08\x12\x16\n\x0e\x63hannel_target\x18\x07 \x01(\t\"\xd3\x01\n\x0bserver_info\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\x16\n\x0eserver_version\x18\x02 \x01(\r\x12\x13\n\x0brpc_version\x18\x03 \x01(\r\x12\x0c\n\x04name\x18\x04 \x01(\t\x12\x0f\n\x07\x63omment\x18\x05 \x01(\t\x12+\n\x07started\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x14\n\x0coption_flags\x18\x07 \x01(\r\x12\x14\n\x0cunique_runid\x18\x08 \x01(\x0c\x32\xcf\x01\n\texptserve\x12Y\n\x15\x42roadcastNotification\x12\x1f.foxtrot.broadcast_notification\x1a\x1f.foxtrot.broadcast_notification\x12\x35\n\rGetServerInfo\x12\x0e.foxtrot.empty\x1a\x14.foxtrot.server_info\x12\x30\n\nDisconnect\x12\x0e.foxtrot.empty\x1a\x12.foxtrot.errstatusB\x0c\n\nio.foxtrotb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'foxtrot_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\nio.foxtrot'
  _globals['_BROADCAST_NOTIFICATION']._serialized_start=92
  _globals['_BROADCAST_NOTIFICATION']._serialized_end=273
  _globals['_SERVER_INFO']._serialized_start=276
  _globals['_SERVER_INFO']._serialized_end=487
  _globals['_EXPTSERVE']._serialized_start=490
  _globals['_EXPTSERVE']._serialized_end=697
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ft_capability.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ft_types_pb2 as ft__types__pb2
import ft_error_pb2 as ft__error__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x13\x66t_capability.proto\x12\x07\x66oxtrot\x1a\x0e\x66t_types.proto\x1a\x0e\x66t_error.proto\x1a\x1fgoogle/protobuf/timestamp.proto\x1a\x1egoogle/protobuf/duration.proto\"\x96\x02\n\rdevcapability\x12%\n\x02tp\x18\x01 \x01(\x0e\x32\x19.foxtrot.capability_types\x12\x0f\n\x07\x63\x61pname\x18\x02 \x01(\t\x12\r\n\x05\x63\x61pid\x18\x08 \x01(\r\x12\x10\n\x08\x61rgnames\x18\x03 \x03(\t\x12-\n\x08\x61rgtypes\x18\x04 \x03(\x0b\x32\x1b.foxtrot.variant_descriptor\x12,\n\x05rettp\x18\x05 \x01(\x0b\x32\x1b.foxtrot.variant_descriptorH\x00\x12,\n\x08vecrettp\x18\x06 \x01(\x0e\x32\x18.foxtrot.byte_data_typesH\x00\x12\x17\n\rdynamic_rettp\x18\x07 \x01(\x08H\x00\x42\x08\n\x06return\"K\n\x13\x63\x61pability_argument\x12\x10\n\x08position\x18\x01 \x01(\r\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.foxtrot.ft_variant\"\x9a\x01\n\x12\x63\x61pability_request\x12\r\n\x05msgid\x18\x01 \x01(\r\x12\r\n\x05\x64\x65vid\x18\x02 \x01(\r\x12\x0f\n\x07\x63\x61pname\x18\x03 \x01(\t\x12\r\n\x05\x63\x61pid\x18\x06 \x01(\r\x12*\n\x04\x61rgs\x18\x04 \x03(\x0b\x32\x1c.foxtrot.capability_argument\x12\x1a\n\x12\x63ontention_timeout\x18\x05 \x01(\r\"\xfd\x01\n\x13\x63\x61pability_response\x12\r\n\x05msgid\x18\x01 \x01(\r\x12\r\n\x05\x64\x65vid\x18\x02 \x01(\r\x12\x0f\n\x07\x63\x61pname\x18\x03 \x01(\t\x12\r\n\x05\x63\x61pid\x18\x07 \x01(\r\x12&\n\treturnval\x18\x04 \x01(\x0b\x32\x13.foxtrot.ft_variant\x12\x1f\n\x03\x65rr\x18\x05 \x01(\x0b\x32\x12.foxtrot.errstatus\x12*\n\x06tstamp\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x33\n\x0ftstamp_dispatch\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\xa8\x01\n\rchunk_request\x12\r\n\x05msgid\x18\x01 \x01(\r\x12\r\n\x05\x64\x65vid\x18\x02 \x01(\r\x12\x0f\n\x07\x63\x61pname\x18\x03 \x01(\t\x12\r\n\x05\x63\x61pid\x18\x08 \x01(\r\x12*\n\x04\x61rgs\x18\x04 \x03(\x0b\x32\x1c.foxtrot.capability_argument\x12\x11\n\tchunksize\x18\x06 \x01(\r\x12\x1a\n\x12\x63ontention_timeout\x18\x07 \x01(\r\"\x9f\x01\n\tdatachunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12\r\n\x05msgid\x18\x02 \x01(\r\x12\r\n\x05\x64\x65vid\x18\x03 \x01(\r\x12\r\n\x05\x63\x61pid\x18\x08 \x01(\r\x12\x0f\n\x07\x63\x61pname\x18\x04 \x01(\t\x12\x1f\n\x03\x65rr\x18\x06 \x01(\x0b\x32\x12.foxtrot.errstatus\x12%\n\x03\x64tp\x18\x07 \x01(\x0e\x32\x18.foxtrot.byte_data_types\"\xa0\x01\n\x0e\x62ulkdata_chunk\x12\x0c\n\x04\x64\x61ta\x18\x01 \x01(\x0c\x12%\n\x03\x64tp\x18\x02 \x01(\x0e\x32\x18.foxtrot.byte_data_types\x12\x1f\n\x03\x65rr\x18\x03 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\x15\n\rcpp_type_name\x18\x04 \x01(\t\x12\x0f\n\x07pagenum\x18\x05 \x01(\r\x12\x10\n\x08totalnum\x18\x06 \x01(\r\"M\n\x10\x62ulkdata_request\x12&\n\x06handle\x18\x01 \x01(\x0b\x32\x16.foxtrot.handlechooser\x12\x11\n\tchunksize\x18\x02 \x01(\r\"g\n\x0b\x64\x65vdescribe\x12\r\n\x05\x64\x65vid\x18\x01 \x01(\r\x12\x0f\n\x07\x64\x65vtype\x18\x02 \x01(\t\x12\x12\n\ndevcomment\x18\x03 \x01(\t\x12$\n\x04\x63\x61ps\x18\x04 \x03(\x0b\x32\x16.foxtrot.devcapability\"\xe0\x01\n\x0cservdescribe\x12\x13\n\x0bservcomment\x18\x01 \x01(\t\x12\x0f\n\x07version\x18\x04 \x01(\t\x12>\n\rdevs_attached\x18\x02 \x03(\x0b\x32\'.foxtrot.servdescribe.DevsAttachedEntry\x12\x1f\n\x03\x65rr\x18\x03 \x01(\x0b\x32\x12.foxtrot.errstatus\x1aI\n\x11\x44\x65vsAttachedEntry\x12\x0b\n\x03key\x18\x01 \x01(\r\x12#\n\x05value\x18\x02 \x01(\x0b\x32\x14.foxtrot.devdescribe:\x02\x38\x01\"\x1e\n\rdevicechooser\x12\r\n\x05\x64\x65vid\x18\x01 \x01(\r\"k\n\x14\x64\x65vice_setup_request\x12\x13\n\x0b\x64\x65vtypename\x18\x01 \x01(\t\x12\x12\n\ndevcomment\x18\x02 \x01(\t\x12*\n\x04\x61rgs\x18\x03 \x03(\x0b\x32\x1c.foxtrot.capability_argument\"[\n\x15\x64\x65vice_setup_response\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12!\n\x03\x64\x65v\x18\x02 \x01(\x0b\x32\x14.foxtrot.devdescribe\"6\n\rhandlechooser\x12\x0e\n\x06handle\x18\x01 \x01(\x04\x12\x15\n\rcpp_type_name\x18\x02 \x01(\t\"5\n\nhandlelist\x12\'\n\x07handles\x18\x01 \x03(\x0b\x32\x16.foxtrot.handlechooser\"\x96\x01\n\x0estream_control\x12(\n\x03req\x18\x01 \x01(\x0b\x32\x1b.foxtrot.capability_request\x12-\n\x03\x63md\x18\x02 \x01(\x0e\x32 .foxtrot.stream_control_commands\x12+\n\x08interval\x18\x03 \x01(\x0b\x32\x19.google.protobuf.Duration\"g\n\rasync_control\x12(\n\x03req\x18\x01 \x01(\x0b\x32\x1b.foxtrot.capability_request\x12,\n\x03\x63md\x18\x02 \x01(\x0e\x32\x1f.foxtrot.async_control_commands\"J\n\x0e\x61sync_response\x12\x0c\n\x04\x64one\x18\x01 \x01(\x08\x12*\n\x04resp\x18\x02 \x01(\x0b\x32\x1c.foxtrot.capability_response*d\n\x10\x63\x61pability_types\x12\x12\n\x0eVALUE_READONLY\x10\x00\x12\x13\n\x0fVALUE_READWRITE\x10\x01\x12\n\n\x06\x41\x43TION\x10\x02\x12\n\n\x06STREAM\x10\x03\x12\x0f\n\x0b\x43ONSTRUCTOR\x10\x04*.\n\x17stream_control_commands\x12\x08\n\x04stop\x10\x00\x12\t\n\x05start\x10\x01*R\n\x16\x61sync_control_commands\x12\x0c\n\x08\x64ispatch\x10\x00\x12\t\n\x05\x63heck\x10\x01\x12\x07\n\x03get\x10\x02\x12\t\n\x05\x64\x65\x66\x65r\x10\x03\x12\x0b\n\x07run_all\x10\x04\x32\xe1\x05\n\ncapability\x12M\n\x10InvokeCapability\x12\x1b.foxtrot.capability_request\x1a\x1c.foxtrot.capability_response\x12\x37\n\x0e\x44\x65scribeServer\x12\x0e.foxtrot.empty\x1a\x15.foxtrot.servdescribe\x12\x39\n\tFetchData\x12\x16.foxtrot.chunk_request\x1a\x12.foxtrot.datachunk0\x01\x12\x39\n\x0b\x43loseDevice\x12\x16.foxtrot.devicechooser\x1a\x12.foxtrot.errstatus\x12\x33\n\rRescanDevices\x12\x0e.foxtrot.empty\x1a\x12.foxtrot.errstatus\x12L\n\x0bSetupDevice\x12\x1d.foxtrot.device_setup_request\x1a\x1e.foxtrot.device_setup_response\x12\x32\n\x0bListHandles\x12\x0e.foxtrot.empty\x1a\x13.foxtrot.handlelist\x12;\n\rDisposeHandle\x12\x16.foxtrot.handlechooser\x1a\x12.foxtrot.errstatus\x12M\n\x10StreamCapability\x12\x17.foxtrot.stream_control\x1a\x1c.foxtrot.capability_response(\x01\x30\x01\x12H\n\x10\x42ulkDataDownload\x12\x19.foxtrot.bulkdata_request\x1a\x17.foxtrot.bulkdata_chunk0\x01\x12H\n\x11InvokeServerAsync\x12\x16.foxtrot.async_control\x1a\x17.foxtrot.async_response(\x01\x30\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ft_capability_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _SERVDESCRIBE_DEVSATTACHEDENTRY._options = None
  _SERVDESCRIBE_DEVSATTACHEDENTRY._serialized_options = b'8\001'
  _globals['_CAPABILITY_TYPES']._serialized_start=2486
  _globals['_CAPABILITY_TYPES']._serialized_end=2586
  _globals['_STREAM_CONTROL_COMMANDS']._serialized_start=2588
  _globals['_STREAM_CONTROL_COMMANDS']._serialized_end=2634
  _globals['_ASYNC_CONTROL_COMMANDS']._serialized_start=2636
  _globals['_ASYNC_CONTROL_COMMANDS']._serialized_end=2718
  _globals['_DEVCAPABILITY']._serialized_start=130
  _globals['_DEVCAPABILITY']._serialized_end=408
  _globals['_CAPABILITY_ARGUMENT']._serialized_start=410
  _globals['_CAPABILITY_ARGUMENT']._serialized_end=485
  _globals['_CAPABILITY_REQUEST']._serialized_start=488
  _globals['_CAPABILITY_REQUEST']._serialized_end=642
  _globals['_CAPABILITY_RESPONSE']._serialized_start=645
  _globals['_CAPABILITY_RESPONSE']._serialized_end=898
  _globals['_CHUNK_REQUEST']._serialized_start=901
  _globals['_CHUNK_REQUEST']._serialized_end=1069
  _globals['_DATACHUNK']._serialized_start=1072
  _globals['_DATACHUNK']._serialized_end=1231
  _globals['_BULKDATA_CHUNK']._serialized_start=1234
  _globals['_BULKDATA_CHUNK']._serialized_end=1394
  _globals['_BULKDATA_REQUEST']._serialized_start=1396
  _globals['_BULKDATA_REQUEST']._serialized_end=1473
  _globals['_DEVDESCRIBE']._serialized_start=1475
  _globals['_DEVDESCRIBE']._serialized_end=1578
  _globals['_SERVDESCRIBE']._serialized_start=1581
  _globals['_SERVDESCRIBE']._serialized_end=1805
  _globals['_SERVDESCRIBE_DEVSATTACHEDENTRY']._serialized_start=1732
  _globals['_SERVDESCRIBE_DEVSATTACHEDENTRY']._serialized_end=1805
  _globals['_DEVICECHOOSER']._serialized_start=1807
  _globals['_DEVICECHOOSER']._serialized_end=1837
  _globals['_DEVICE_SETUP_REQUEST']._serialized_start=1839
  _globals['_DEVICE_SETUP_REQUEST']._serialized_end=1946
  _globals['_DEVICE_SETUP_RESPONSE']._serialized_start=1948
  _globals['_DEVICE_SETUP_RESPONSE']._serialized_end=2039
  _globals['_HANDLECHOOSER']._serialized_start=2041
  _globals['_HANDLECHOOSER']._serialized_end=2095
  _globals['_HANDLELIST']._serialized_start=2097
  _globals['_HANDLELIST']._serialized_end=2150
  _globals['_STREAM_CONTROL']._serialized_start=2153
  _globals['_STREAM_CONTROL']._serialized_end=2303
  _globals['_ASYNC_CONTROL']._serialized_start=2305
  _globals['_ASYNC_CONTROL']._serialized_end=2408
  _globals['_ASYNC_RESPONSE']._serialized_start=2410
  _globals['_ASYNC_RESPONSE']._serialized_end=2484
  _globals['_CAPABILITY']._serialized_start=2721
  _globals['_CAPABILITY']._serialized_end=3458
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import ft_sessions_pb2 as ft__sessions__pb2
import ft_types_pb2 as ft__types__pb2


class sessionsStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.StartSession = channel.unary_unary(
                '/foxtrot.sessions/StartSession',
                request_serializer=ft__sessions__pb2.session_info.SerializeToString,
                response_deserializer=ft__sessions__pb2.session_info.FromString,
                )
        self.CloseSession = channel.unary_unary(
                '/foxtrot.sessions/CloseSession',
                request_serializer=ft__sessions__pb2.session_info.SerializeToString,
                response_deserializer=ft__sessions__pb2.session_info.FromString,
                )
        self.KeepAliveSession = channel.unary_unary(
                '/foxtrot.sessions/KeepAliveSession',
                request_serializer=ft__sessions__pb2.session_info.SerializeToString,
                response_deserializer=ft__sessions__pb2.session_info.FromString,
                )
        self.ListSessions = channel.unary_unary(
                '/foxtrot.sessions/ListSessions',
                request_serializer=ft__types__pb2.empty.SerializeToString,
                response_deserializer=ft__sessions__pb2.session_list.FromString,
                )


class sessionsServicer(object):
    """Missing associated documentation comment in .proto file."""

    def StartSession(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseSession(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def KeepAliveSession(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListSessions(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_sessionsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'StartSession': grpc.unary_unary_rpc_method_handler(
                    servicer.StartSession,
                    request_deserializer=ft__sessions__pb2.session_info.FromString,
                    response_serializer=ft__sessions__pb2.session_info.SerializeToString,
            ),
            'CloseSession': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseSession,
                    request_deserializer=ft__sessions__pb2.session_info.FromString,
                    response_serializer=ft__sessions__pb2.session_info.SerializeToString,
            ),
            'KeepAliveSession': grpc.unary_unary_rpc_method_handler(
                    servicer.KeepAliveSession,
                    request_deserializer=ft__sessions__pb2.session_info.FromString,
                    response_serializer=ft__sessions__pb2.session_info.SerializeToString,
            ),
            'ListSessions': grpc.unary_unary_rpc_method_handler(
                    servicer.ListSessions,
                    request_deserializer=ft__types__pb2.empty.FromString,
                    response_serializer=ft__sessions__pb2.session_list.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'foxtrot.sessions', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class sessions(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def StartSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.sessions/StartSession',
            ft__sessions__pb2.session_info.SerializeToString,
            ft__sessions__pb2.session_info.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.sessions/CloseSession',
            ft__sessions__pb2.session_info.SerializeToString,
            ft__sessions__pb2.session_info.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def KeepAliveSession(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.sessions/KeepAliveSession',
            ft__sessions__pb2.session_info.SerializeToString,
            ft__sessions__pb2.session_info.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListSessions(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.sessions/ListSessions',
            ft__types__pb2.empty.SerializeToString,
            ft__sessions__pb2.session_list.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# -*- coding: utf-8 -*-

import base64
from pyfoxtrot.Errors import DeviceError, ProtocolError, ContentionError, ServerError, AuthenticationError

def _check_repl_err(repl):
    if not repl.HasField('err'):
        return repl

    if repl.err.tp == 0:
        raise RuntimeError(repl.err.msg)
    elif repl.err.tp == 1:
        raise DeviceError(repl.err.msg, repl)
    elif repl.err.tp == 2:
        raise ProtocolError(repl.err.msg, repl)
    elif repl.err.tp == 3:
        raise ValueError(repl.err.msg)
    elif repl.err.tp == 5:
        raise ContentionError(repl.err.msg, repl)
    elif repl.err.tp == 6:
        raise ServerError(repl.err.msg, repl)
    elif repl.err.tp == 7:
        raise AuthenticationError(repl.err.msg, repl)
    else:
        raise RuntimeError("unknown error")

def decode_sodiumkey(key_in: str) -> bytes:
    return base64.standard_b64decode(key_in + "=====")

def encode_sodiumkey(key_in: str) -> bytes:
    return base64.standard_b64encode(key_in).rstrip(b"=")


import ft_error_pb2 as _ft_error_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
import ft_types_pb2 as _ft_types_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class session_info(_message.Message):
    __slots__ = ["err", "sessionid", "user_identifier", "comment", "devices", "flags", "expiry"]
    ERR_FIELD_NUMBER: _ClassVar[int]
    SESSIONID_FIELD_NUMBER: _ClassVar[int]
    USER_IDENTIFIER_FIELD_NUMBER: _ClassVar[int]
    COMMENT_FIELD_NUMBER: _ClassVar[int]
    DEVICES_FIELD_NUMBER: _ClassVar[int]
    FLAGS_FIELD_NUMBER: _ClassVar[int]
    EXPIRY_FIELD_NUMBER: _ClassVar[int]
    err: _ft_error_pb2.errstatus
    sessionid: bytes
    user_identifier: str
    comment: str
    devices: _containers.RepeatedScalarFieldContainer[int]
    flags: _containers.RepeatedScalarFieldContainer[str]
    expiry: _timestamp_pb2.Timestamp
    def __init__(self, err: _Optional[_Union[_ft_error_pb2.errstatus, _Mapping]] = ..., sessionid: _Optional[bytes] = ..., user_identifier: _Optional[str] = ..., comment: _Optional[str] = ..., devices: _Optional[_Iterable[int]] = ..., flags: _Optional[_Iterable[str]] = ..., expiry: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class session_list(_message.Message):
    __slots__ = ["err", "sessions"]
    ERR_FIELD_NUMBER: _ClassVar[int]
    SESSIONS_FIELD_NUMBER: _ClassVar[int]
    err: _ft_error_pb2.errstatus
    sessions: _containers.RepeatedCompositeFieldContainer[session_info]
    def __init__(self, err: _Optional[_Union[_ft_error_pb2.errstatus, _Mapping]] = ..., sessions: _Optional[_Iterable[_Union[session_info, _Mapping]]] = ...) -> None: ...



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ft_types.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0e\x66t_types.proto\x12\x07\x66oxtrot\x1a\x1fgoogle/protobuf/timestamp.proto\"\x07\n\x05\x65mpty\"\xb7\x03\n\x12variant_descriptor\x12,\n\x0cvariant_type\x18\x01 \x01(\x0e\x32\x16.foxtrot.variant_types\x12\x34\n\x10simplevalue_type\x18\x02 \x01(\x0e\x32\x1a.foxtrot.simplevalue_types\x12\x1a\n\x12simplevalue_sizeof\x18\x03 \x01(\r\x12/\n\x0bstruct_desc\x18\x04 \x01(\x0b\x32\x1a.foxtrot.struct_descriptor\x12+\n\tenum_desc\x18\x05 \x01(\x0b\x32\x18.foxtrot.enum_descriptor\x12-\n\ntuple_desc\x18\x06 \x01(\x0b\x32\x19.foxtrot.tuple_descriptor\x12\x39\n\x10homog_array_desc\x18\x07 \x01(\x0b\x32\x1f.foxtrot.homog_array_descriptor\x12-\n\nunion_desc\x18\t \x01(\x0b\x32\x19.foxtrot.union_descriptor\x12\x15\n\rcpp_type_name\x18\x08 \x01(\t\x12\x13\n\x0bis_nullable\x18\n \x01(\x08\"\xb6\x01\n\x11struct_descriptor\x12\x13\n\x0bstruct_name\x18\x01 \x01(\t\x12=\n\nstruct_map\x18\x02 \x03(\x0b\x32).foxtrot.struct_descriptor.StructMapEntry\x1aM\n\x0eStructMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12*\n\x05value\x18\x02 \x01(\x0b\x32\x1b.foxtrot.variant_descriptor:\x02\x38\x01\"\x8d\x01\n\x0f\x65num_descriptor\x12\x11\n\tenum_name\x18\x01 \x01(\t\x12\x37\n\x08\x65num_map\x18\x02 \x03(\x0b\x32%.foxtrot.enum_descriptor.EnumMapEntry\x1a.\n\x0c\x45numMapEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\r:\x02\x38\x01\"B\n\x10tuple_descriptor\x12.\n\ttuple_map\x18\x01 \x03(\x0b\x32\x1b.foxtrot.variant_descriptor\"u\n\x16homog_array_descriptor\x12\x16\n\x0ehas_fixed_size\x18\x03 \x01(\x08\x12\x12\n\nfixed_size\x18\x04 \x01(\x04\x12/\n\nvalue_type\x18\x06 \x01(\x0b\x32\x1b.foxtrot.variant_descriptor\"G\n\x10union_descriptor\x12\x33\n\x0epossible_types\x18\x01 \x03(\x0b\x32\x1b.foxtrot.variant_descriptor\"\xe7\x01\n\nft_variant\x12.\n\tsimplevar\x18\x01 \x01(\x0b\x32\x19.foxtrot.ft_simplevariantH\x00\x12\'\n\tstructval\x18\x02 \x01(\x0b\x32\x12.foxtrot.ft_structH\x00\x12#\n\x07\x65numval\x18\x03 \x01(\x0b\x32\x10.foxtrot.ft_enumH\x00\x12%\n\x08tupleval\x18\x04 \x01(\x0b\x32\x11.foxtrot.ft_tupleH\x00\x12+\n\x08\x61rrayval\x18\x05 \x01(\x0b\x32\x17.foxtrot.ft_homog_arrayH\x00\x42\x07\n\x05value\"\xdf\x01\n\x10\x66t_simplevariant\x12\x10\n\x06\x64\x62lval\x18\x01 \x01(\x01H\x00\x12\x10\n\x06intval\x18\x02 \x01(\x12H\x00\x12\x11\n\x07uintval\x18\x03 \x01(\x04H\x00\x12\x11\n\x07\x62oolval\x18\x04 \x01(\x08H\x00\x12\x13\n\tstringval\x18\x05 \x01(\tH\x00\x12/\n\ttstampval\x18\x08 \x01(\x0b\x32\x1a.google.protobuf.TimestampH\x00\x12\x13\n\thandleval\x18\t \x01(\x04H\x00\x12\x0c\n\x04size\x18\x06 \x01(\r\x12\x0f\n\x07is_void\x18\x07 \x01(\x08\x42\x07\n\x05value\"\x91\x01\n\tft_struct\x12\x13\n\x0bstruct_name\x18\x01 \x01(\t\x12,\n\x05value\x18\x02 \x03(\x0b\x32\x1d.foxtrot.ft_struct.ValueEntry\x1a\x41\n\nValueEntry\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\"\n\x05value\x18\x02 \x01(\x0b\x32\x13.foxtrot.ft_variant:\x02\x38\x01\"E\n\x07\x66t_enum\x12&\n\x04\x64\x65sc\x18\x01 \x01(\x0b\x32\x18.foxtrot.enum_descriptor\x12\x12\n\nenum_value\x18\x02 \x01(\r\".\n\x08\x66t_tuple\x12\"\n\x05value\x18\x02 \x03(\x0b\x32\x13.foxtrot.ft_variant\"\xb7\x01\n\x0e\x66t_homog_array\x12\x36\n\x0b\x61rr_decoded\x18\x01 \x01(\x0b\x32\x1f.foxtrot.ft_homog_array_decodedH\x00\x12\x36\n\x0b\x61rr_encoded\x18\x02 \x01(\x0b\x32\x1f.foxtrot.ft_homog_array_encodedH\x00\x12,\n\tarr_heavy\x18\x03 \x01(\x0b\x32\x17.foxtrot.ft_heavy_arrayH\x00\x42\x07\n\x05\x61rray\"M\n\x16\x66t_homog_array_encoded\x12%\n\x03\x64tp\x18\x01 \x01(\x0e\x32\x18.foxtrot.byte_data_types\x12\x0c\n\x04\x64\x61ta\x18\x02 \x01(\x0c\"`\n\x0f\x66t_dynamic_type\x12\"\n\x05value\x18\x01 \x01(\x0b\x32\x13.foxtrot.ft_variant\x12)\n\x04\x64\x65sc\x18\x02 \x01(\x0b\x32\x1b.foxtrot.variant_descriptor\"3\n\x0e\x66t_heavy_array\x12!\n\x04\x64\x61ta\x18\x01 \x03(\x0b\x32\x13.foxtrot.ft_variant\"\xbb\x03\n\x16\x66t_homog_array_decoded\x12*\n\tarr_uint8\x18\x01 \x01(\x0b\x32\x15.foxtrot.ft_uint8_arrH\x00\x12,\n\narr_uint16\x18\x02 \x01(\x0b\x32\x16.foxtrot.ft_uint32_arrH\x00\x12,\n\narr_sint16\x18\x03 \x01(\x0b\x32\x16.foxtrot.ft_sint32_arrH\x00\x12,\n\narr_uint32\x18\x04 \x01(\x0b\x32\x16.foxtrot.ft_uint32_arrH\x00\x12,\n\narr_sint32\x18\x05 \x01(\x0b\x32\x16.foxtrot.ft_sint32_arrH\x00\x12,\n\narr_uint64\x18\x06 \x01(\x0b\x32\x16.foxtrot.ft_uint64_arrH\x00\x12,\n\narr_sint64\x18\x07 \x01(\x0b\x32\x16.foxtrot.ft_sint64_arrH\x00\x12*\n\tarr_float\x18\x08 \x01(\x0b\x32\x15.foxtrot.ft_float_arrH\x00\x12,\n\narr_double\x18\t \x01(\x0b\x32\x16.foxtrot.ft_double_arrH\x00\x42\x07\n\x05\x61rray\"\x1e\n\x0c\x66t_uint8_arr\x12\x0e\n\x06values\x18\x01 \x03(\r\"\x1f\n\rft_uint16_arr\x12\x0e\n\x06values\x18\x01 \x03(\r\"\x1f\n\rft_sint16_arr\x12\x0e\n\x06values\x18\x01 \x03(\x11\"\x1f\n\rft_uint32_arr\x12\x0e\n\x06values\x18\x01 \x03(\r\"\x1f\n\rft_sint32_arr\x12\x0e\n\x06values\x18\x01 \x03(\x11\"\x1f\n\rft_uint64_arr\x12\x0e\n\x06values\x18\x01 \x03(\x04\"\x1f\n\rft_sint64_arr\x12\x0e\n\x06values\x18\x01 \x03(\x12\"\x1e\n\x0c\x66t_float_arr\x12\x0e\n\x06values\x18\x01 \x03(\x02\"\x1f\n\rft_double_arr\x12\x0e\n\x06values\x18\x01 \x03(\x01*\x9e\x01\n\x11simplevalue_types\x12\x0e\n\nFLOAT_TYPE\x10\x00\x12\x0c\n\x08INT_TYPE\x10\x01\x12\x11\n\rUNSIGNED_TYPE\x10\x02\x12\r\n\tBOOL_TYPE\x10\x03\x12\x0f\n\x0bSTRING_TYPE\x10\x04\x12\r\n\tVOID_TYPE\x10\x05\x12\x11\n\rDATETIME_TYPE\x10\x06\x12\x16\n\x12REMOTE_HANDLE_TYPE\x10\x07*\xea\x01\n\x0f\x62yte_data_types\x12\x0e\n\nUCHAR_TYPE\x10\x00\x12\r\n\tCHAR_TYPE\x10\x01\x12\x0f\n\x0bUSHORT_TYPE\x10\x02\x12\r\n\tUINT_TYPE\x10\x03\x12\x0e\n\nULONG_TYPE\x10\x04\x12\x0e\n\nSHORT_TYPE\x10\x05\x12\r\n\tIINT_TYPE\x10\x06\x12\r\n\tLONG_TYPE\x10\x07\x12\x0f\n\x0b\x42\x46LOAT_TYPE\x10\x08\x12\x10\n\x0c\x42\x44OUBLE_TYPE\x10\t\x12\x11\n\rBCOMPLEX_TYPE\x10\x0b\x12\x12\n\x0e\x42\x44\x41TETIME_TYPE\x10\x0c\x12\x10\n\x0c\x42OPAQUE_TYPE\x10\r*\x8b\x01\n\rvariant_types\x12\x12\n\x0eSIMPLEVAR_TYPE\x10\x00\x12\x0f\n\x0bSTRUCT_TYPE\x10\x01\x12\r\n\tENUM_TYPE\x10\x02\x12\x0e\n\nTUPLE_TYPE\x10\x03\x12\x14\n\x10HOMOG_ARRAY_TYPE\x10\x04\x12\x0e\n\nUNION_TYPE\x10\x05\x12\x10\n\x0c\x44YNAMIC_TYPE\x10\x06\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ft_types_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _STRUCT_DESCRIPTOR_STRUCTMAPENTRY._options = None
  _STRUCT_DESCRIPTOR_STRUCTMAPENTRY._serialized_options = b'8\001'
  _ENUM_DESCRIPTOR_ENUMMAPENTRY._options = None
  _ENUM_DESCRIPTOR_ENUMMAPENTRY._serialized_options = b'8\001'
  _FT_STRUCT_VALUEENTRY._options = None
  _FT_STRUCT_VALUEENTRY._serialized_options = b'8\001'
  _globals['_SIMPLEVALUE_TYPES']._serialized_start=2985
  _globals['_SIMPLEVALUE_TYPES']._serialized_end=3143
  _globals['_BYTE_DATA_TYPES']._serialized_start=3146
  _globals['_BYTE_DATA_TYPES']._serialized_end=3380
  _globals['_VARIANT_TYPES']._serialized_start=3383
  _globals['_VARIANT_TYPES']._serialized_end=3522
  _globals['_EMPTY']._serialized_start=60
  _globals['_EMPTY']._serialized_end=67
  _globals['_VARIANT_DESCRIPTOR']._serialized_start=70
  _globals['_VARIANT_DESCRIPTOR']._serialized_end=509
  _globals['_STRUCT_DESCRIPTOR']._serialized_start=512
  _globals['_STRUCT_DESCRIPTOR']._serialized_end=694
  _globals['_STRUCT_DESCRIPTOR_STRUCTMAPENTRY']._serialized_start=617
  _globals['_STRUCT_DESCRIPTOR_STRUCTMAPENTRY']._serialized_end=694
  _globals['_ENUM_DESCRIPTOR']._serialized_start=697
  _globals['_ENUM_DESCRIPTOR']._serialized_end=838
  _globals['_ENUM_DESCRIPTOR_ENUMMAPENTRY']._serialized_start=792
  _globals['_ENUM_DESCRIPTOR_ENUMMAPENTRY']._serialized_end=838
  _globals['_TUPLE_DESCRIPTOR']._serialized_start=840
  _globals['_TUPLE_DESCRIPTOR']._serialized_end=906
  _globals['_HOMOG_ARRAY_DESCRIPTOR']._serialized_start=908
  _globals['_HOMOG_ARRAY_DESCRIPTOR']._serialized_end=1025
  _globals['_UNION_DESCRIPTOR']._serialized_start=1027
  _globals['_UNION_DESCRIPTOR']._serialized_end=1098
  _globals['_FT_VARIANT']._serialized_start=1101
  _globals['_FT_VARIANT']._serialized_end=1332
  _globals['_FT_SIMPLEVARIANT']._serialized_start=1335
  _globals['_FT_SIMPLEVARIANT']._serialized_end=1558
  _globals['_FT_STRUCT']._serialized_start=1561
  _globals['_FT_STRUCT']._serialized_end=1706
  _globals['_FT_STRUCT_VALUEENTRY']._serialized_start=1641
  _globals['_FT_STRUCT_VALUEENTRY']._serialized_end=1706
  _globals['_FT_ENUM']._serialized_start=1708
  _globals['_FT_ENUM']._serialized_end=1777
  _globals['_FT_TUPLE']._serialized_start=1779
  _globals['_FT_TUPLE']._serialized_end=1825
  _globals['_FT_HOMOG_ARRAY']._serialized_start=1828
  _globals['_FT_HOMOG_ARRAY']._serialized_end=2011
  _globals['_FT_HOMOG_ARRAY_ENCODED']._serialized_start=2013
  _globals['_FT_HOMOG_ARRAY_ENCODED']._serialized_end=2090
  _globals['_FT_DYNAMIC_TYPE']._serialized_start=2092
  _globals['_FT_DYNAMIC_TYPE']._serialized_end=2188
  _globals['_FT_HEAVY_ARRAY']._serialized_start=2190
  _globals['_FT_HEAVY_ARRAY']._serialized_end=2241
  _globals['_FT_HOMOG_ARRAY_DECODED']._serialized_start=2244
  _globals['_FT_HOMOG_ARRAY_DECODED']._serialized_end=2687
  _globals['_FT_UINT8_ARR']._serialized_start=2689
  _globals['_FT_UINT8_ARR']._serialized_end=2719
  _globals['_FT_UINT16_ARR']._serialized_start=2721
  _globals['_FT_UINT16_ARR']._serialized_end=2752
  _globals['_FT_SINT16_ARR']._serialized_start=2754
  _globals['_FT_SINT16_ARR']._serialized_end=2785
  _globals['_FT_UINT32_ARR']._serialized_start=2787
  _globals['_FT_UINT32_ARR']._serialized_end=2818
  _globals['_FT_SINT32_ARR']._serialized_start=2820
  _globals['_FT_SINT32_ARR']._serialized_end=2851
  _globals['_FT_UINT64_ARR']._serialized_start=2853
  _globals['_FT_UINT64_ARR']._serialized_end=2884
  _globals['_FT_SINT64_ARR']._serialized_start=2886
  _globals['_FT_SINT64_ARR']._serialized_end=2917
  _globals['_FT_FLOAT_ARR']._serialized_start=2919
  _globals['_FT_FLOAT_ARR']._serialized_end=2949
  _globals['_FT_DOUBLE_ARR']._serialized_start=2951
  _globals['_FT_DOUBLE_ARR']._serialized_end=2982
# @@protoc_insertion_point(module_scope)



import grpc

from pyfoxtrot.ft_sessions_pb2_grpc import sessionsStub
from pyfoxtrot.ft_sessions_pb2 import session_info, session_list
from pyfoxtrot.ft_types_pb2 import empty
from pyfoxtrot.Client import Client
from pyfoxtrot.common import _check_repl_err, decode_sodiumkey, encode_sodiumkey
from pyfoxtrot.Errors import ServerError
from datetime import datetime, timedelta
from time import sleep

class SessionConflictError(ServerError):
    @classmethod
    def _from_raw_error(cls, err: ServerError):
        obj = cls(*err.args)
        obj.owner = err.raw_reply.user_identifier
        obj.comment = err.raw_reply.comment
        return obj

class Session:
    @classmethod
    def _from_repl(cls, repl: session_info, cl: Client = None, sm= None,
                   interactive: bool=True):
        obj = cls()
        obj._userid = repl.user_identifier
        obj._comment = repl.comment
        obj._secret = repl.sessionid
        obj._expiry = datetime.fromtimestamp(repl.expiry.seconds)
        obj.raw_repl = repl
        obj._devices = repl.devices
        obj._flags = repl.flags
        obj._cl = cl
        obj._sm = sm
        obj._interactive = interactive
        return obj
    
    def close(self):
        if self._sm is None:
            raise ValueError("this session object doesn't have a session manager attached")
        self._sm._close_session(self)
    
    @property
    def userid(self):
        return self._userid
    
    @property
    def comment(self):
        return self._comment
    
    @property
    def expiry(self):
        return self._expiry
    
    @property
    def devices(self):
        if self._cl is not None:
            return [self._cl[_] for _ in self._devices]
        else:
            return self._devices
    
    @property
    def flags(self):
        if self._cl is not None:
            return [self._cl.flags[_] for _ in self._flags]
        else:
            return self._flags
    
    @property
    def isexpired(self) -> bool:
        now = datetime.now()
        return self.expiry < now
    
    @property
    def isowned(self) -> bool:
        return len(self._secret) > 0
    
    def __repr__(self) -> str:
        isexpiredstr = "EXPIRED" if self.isexpired else "ACTIVE"
        
        ownedstr = "OWNER" if self.isowned else "INFO"
        
        strtime = self.expiry.strftime("%c")
        return "Session(user=%s, comment='%s', expiry='%s') - %s, %s"  % \
                  (self.userid, self.comment, strtime, isexpiredstr, ownedstr)

    def __del__(self):
        if self._interactive:
            print("closing interactive session")
            self.close()

class SessionManager:
    def __init__(self, cl: Client):
        self._stub = sessionsStub(cl._channel)
        self._cl = cl
    
    def _start_session(self, userid: str, comment: str, devices=None,
                        flags=None, expiry_request: datetime=None) -> Session:
        req = session_info(user_identifier=userid,
                           comment=comment)
        
        if devices is not None:
            req.devices.extend([_._devid for _ in devices])
        if flags is not None:
            req.flags.extend([_._flagname for _ in flags])
        if expiry_request is not None:
            req.expiry.seconds = int(expiry_request.timestamp())
        
        repl = self._stub.StartSession(req)
        
        try:
            _check_repl_err(repl)
        except ServerError as err:
            sce = SessionConflictError._from_raw_error(err)
            raise sce
        
        ses = Session._from_repl(repl, self._cl, self)
        self._cl._active_session = ses
        return ses

    def _close_session(self, session: Session):
        req = session_info(sessionid = session._secret)
        
        repl = self._stub.CloseSession(req)
        _check_repl_err(repl)
        if self._cl._active_session is session:
            self._cl._active_session = None

    def _list_sessions(self):
        req = empty()
        repl = self._stub.ListSessions(req)
        _check_repl_err(repl)
        
        return [Session._from_repl(_,self._cl, self) for _ in repl.sessions]

    def _renew_session(self, ses: Session, requested_expiry: datetime = None):
        req = session_info(sessionid = ses._secret)
        if requested_expiry is not None:
            req.expiry.seconds = int(requested_expiry.timestamp())
        
        repl = self._stub.KeepAliveSession(req)
        _check_repl_err(repl)
        ses._expiry = datetime.fromtimestamp(repl.expiry.seconds)

if __name__ == "__main__":
    cl = Client("localhost:50051")
    
    sm = SessionManager(cl)
    ses = sm._start_session("danw", "comment1", [cl.dummy1])
    print(ses)
#    sleep(1)
#    print("trying to close session..")
#    sm._close_session(ses)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Jul 18 12:54:38 2019

@author: weatherill
"""

from enum import IntEnum
from .protos.ft_types_pb2 import enum_descriptor

def define_enum(desc: enum_descriptor):
    newtypename = desc.enum_name.replace("::","_")
    tp = IntEnum(newtypename, dict(desc.enum_map.items()))

    return tp



from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class error_types(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    ft_Error: _ClassVar[error_types]
    ft_DeviceError: _ClassVar[error_types]
    ft_ProtocolError: _ClassVar[error_types]
    out_of_range: _ClassVar[error_types]
    unknown_error: _ClassVar[error_types]
    contention_timeout: _ClassVar[error_types]
    ft_ServerError: _ClassVar[error_types]
    ft_AuthError: _ClassVar[error_types]
ft_Error: error_types
ft_DeviceError: error_types
ft_ProtocolError: error_types
out_of_range: error_types
unknown_error: error_types
contention_timeout: error_types
ft_ServerError: error_types
ft_AuthError: error_types

class errstatus(_message.Message):
    __slots__ = ["tp", "msg"]
    TP_FIELD_NUMBER: _ClassVar[int]
    MSG_FIELD_NUMBER: _ClassVar[int]
    tp: error_types
    msg: str
    def __init__(self, tp: _Optional[_Union[error_types, str]] = ..., msg: _Optional[str] = ...) -> None: ...



from .protos.ft_types_pb2 import ft_variant, ft_simplevariant, ft_enum, ft_struct, ft_homog_array
from .protos.ft_types_pb2 import simplevalue_types, variant_types
from .protos.ft_types_pb2 import variant_descriptor, struct_descriptor, enum_descriptor, homog_array_descriptor
from .protos.ft_types_pb2 import tuple_descriptor, ft_tuple
from .protos.ft_types_pb2 import ENUM_TYPE, STRUCT_TYPE, SIMPLEVAR_TYPE, TUPLE_TYPE, UNION_TYPE, HOMOG_ARRAY_TYPE
from .protos.ft_types_pb2 import INT_TYPE, UNSIGNED_TYPE, BOOL_TYPE, STRING_TYPE, VOID_TYPE, FLOAT_TYPE

from .protos.ft_types_pb2 import UCHAR_TYPE, CHAR_TYPE, USHORT_TYPE, UINT_TYPE, ULONG_TYPE, SHORT_TYPE, IINT_TYPE, LONG_TYPE, BFLOAT_TYPE, BDOUBLE_TYPE
import struct
import warnings
from typing import Iterable


def get_struct_string(dtp, length: int) -> str:
    if dtp == UCHAR_TYPE:
        structstr = '<%dB' % length
    elif dtp == CHAR_TYPE:
        structstr = '<%db' % length
    elif dtp == USHORT_TYPE:
        structstr = '<%dH' % (length // 2)
    elif dtp == UINT_TYPE:
        structstr = '<%dI' % (length // 4)
    elif dtp == IINT_TYPE:
        structstr = '<%di' % (length // 4)
    elif dtp == BDOUBLE_TYPE:
        structstr = '<%dd' % (length // 8)
    elif dtp == BFLOAT_TYPE:
        structstr = '<%df' % (length // 4)
    else:
        #NOTE: all the others are dodgy at the moment
        raise NotImplementedError("data type not supported yet")

    return structstr


def ft_variant_from_value(val, descriptor: variant_descriptor) -> ft_variant:
    vartype = descriptor.variant_type
    out = ft_variant()
    if vartype == SIMPLEVAR_TYPE:
        out.simplevar.CopyFrom(ft_simplevariant_from_value(val, descriptor))
    elif vartype == STRUCT_TYPE:
        out.structval.CopyFrom(ft_struct_from_value(val, descriptor))
    elif vartype == ENUM_TYPE:
        out.enumval.CopyFrom(ft_enum_from_value(val, descriptor))
    elif vartype == UNION_TYPE:
        out = ft_union_from_value(val, descriptor)
    elif vartype == TUPLE_TYPE:
        out.tupleval.CopyFrom(ft_tuple_from_value(val, descriptor.tuple_desc))
    else:
        raise RuntimeError("couldn't determine type from descriptor")
    return out


def _ensure_type(val, tp: type, cond=None, errmsg=None):
    if not isinstance(val, tp):
        raise TypeError("method requires %s value" % tp.__name__)
    if cond is not None:
        if not cond(val):
            raise ValueError("Value failed condition: %s" % errmsg)


def ft_simplevariant_from_value(val, descriptor: variant_descriptor):
    out = ft_simplevariant()
    out.size = descriptor.simplevalue_sizeof
    svt = descriptor.simplevalue_type
    if svt == simplevalue_types.Value("FLOAT_TYPE"):
        _ensure_type(val, float)
        out.dblval = val
    elif svt == simplevalue_types.Value("INT_TYPE"):
        _ensure_type(val, int)
        out.intval = val
    elif svt == simplevalue_types.Value("UNSIGNED_TYPE"):
        _ensure_type(val, int, lambda s: s >= 0, "must be unsigned")
        out.uintval = val
    elif svt == simplevalue_types.Value("BOOL_TYPE"):
        _ensure_type(val, bool)
        out.boolval = val
    elif svt == simplevalue_types.Value("STRING_TYPE"):
        _ensure_type(val, str)
        out.stringval = val.encode("UTF-8")
    elif svt ==simplevalue_types.Value("REMOTE_HANDLE_TYPE"):
        _ensure_type(val, int)
        out.handleval = val
    elif svt == simplevalue_types.Value("VOID_TYPE"):
        raise TypeError("cannot assign to void type")
    else:
        raise RuntimeError("failed to convert, maybe a problem in descriptor")

    return out

def ft_tuple_from_value(val: tuple, descriptor: tuple_descriptor) -> ft_tuple:
    out = ft_tuple()
    if not isinstance(val, Iterable):
        raise TypeError("value must be an iterable")

    if len(val) != len(descriptor.tuple_map):
        raise ValueError(f"invalid tuple length {len(val)} supplied, requested length is {len(descriptor.tuple_map)}")
    
    for innerval, innerdesc in zip(val, descriptor.tuple_map):
        out.value.append(ft_variant_from_value(innerval, innerdesc))

    return out



def ft_struct_from_value(val: dict,
                         descriptor: variant_descriptor) -> ft_struct:
    out = ft_struct()
    _ensure_type(val, dict,
                 lambda s: all(isinstance(_, str) for _ in s.keys()),
                 "all keys must of string type")
    
    out.struct_name = descriptor.struct_desc.struct_name
    for nm, desc in descriptor.struct_desc.struct_map.items():
        if nm not in val:
            raise NameError("missing key: %s in input struct" % nm)
        out.value[nm].MergeFrom(ft_variant_from_value(val[nm], desc))
    
    return out

def ft_enum_from_value(val, descriptor: variant_descriptor) -> ft_enum:
    out = ft_enum(desc=descriptor.enum_desc)
    if isinstance(val, str):
        if val not in descriptor.enum_desc.enum_map:
            raise KeyError("couldn't map string %s to enum value" % val)
        sanval = descriptor.enum_desc.enum_map[val]
    elif isinstance(val, int):
        if int(val) not in descriptor.enum_desc.enum_map.values():
            raise KeyError("couldn't map int %d to valid enum value" % val)
        sanval = val
    else:
        raise RuntimeError("type: %s is not supported for converting to enum yet"
                           % type(val).__name__)

    out.enum_value = sanval
    return out

def ft_union_from_value(val,
                        descriptor: variant_descriptor) -> ft_variant:

    for possible_type in descriptor.union_desc.possible_types:
        try:
            out = ft_variant_from_value(val, possible_type)
            return out
        except Exception as err:
            continue
    else:
        raise RuntimeError("failed to convert union value!")



def value_from_ft_variant(variant):
    whichattr = variant.WhichOneof("value")
    if whichattr == "simplevar":
        return value_from_ft_simplevar(variant.simplevar)
    elif whichattr == "structval":
        return value_from_ft_struct(variant.structval)
    elif whichattr == "enumval":
        return value_from_ft_enum(variant.enumval)
    elif whichattr == "tupleval":
        return value_from_ft_tuple(variant.tupleval)
    elif whichattr == "arrayval":
        return value_from_ft_array(variant.arrayval)
    else:
        raise ValueError("couldn't determine variant type")


def value_from_ft_array(variant: ft_homog_array):
    whichattr = variant.WhichOneof("array")
    if whichattr == "arr_decoded":
        raise NotImplementedError("don't understand decoded arrays yet")
    elif whichattr == "arr_heavy":
        heavy_arr = variant.arr_heavy.data
        return [value_from_ft_variant(_) for _ in heavy_arr]
    else:
        rawdat = variant.arr_encoded.data
        dtp = variant.arr_encoded.dtp
        structstr = get_struct_string(dtp, len(rawdat))
        return struct.unpack(structstr, rawdat)

def value_from_ft_simplevar(variant: ft_simplevariant):
    whichattr = variant.WhichOneof("value")
    if whichattr is not None:
        return getattr(variant, whichattr)
    else:
        return None


def value_from_ft_struct(variant: ft_struct):
    return {k: value_from_ft_variant(v) for k, v in variant.value.items()}


def value_from_ft_enum(variant: ft_enum):
    return variant.enum_value

def value_from_ft_tuple(variant: ft_tuple):
    v = variant.value
    return tuple( value_from_ft_variant(_) for _ in v)

_simplevar_stringdescs_py_style = {(FLOAT_TYPE, 4): "float[4]",
                                   (FLOAT_TYPE, 8): "float[8]",
                                   (INT_TYPE, 4): "int[4]",
                                   (INT_TYPE, 8): "int[8]",
                                   (INT_TYPE, 2): "int[2]",
                                   (INT_TYPE, 1): "int[1]",
                                   (UNSIGNED_TYPE, 1): "uint[1]",
                                   (UNSIGNED_TYPE, 2): "uint[2]",
                                   (UNSIGNED_TYPE, 4): "uint[4]",
                                   (UNSIGNED_TYPE, 8): "uint[8]",
                                   BOOL_TYPE:  "bool",
                                   VOID_TYPE: "void",
                                   STRING_TYPE: "str"}

def _string_describe_simple_variant(descriptor):
    tp = descriptor.simplevalue_type
    size = descriptor.simplevalue_sizeof

    if tp in (BOOL_TYPE, VOID_TYPE, STRING_TYPE):
        typestr = _simplevar_stringdescs_py_style[tp]
    elif (tp, size) in _simplevar_stringdescs_py_style:
        typestr = _simplevar_stringdescs_py_style[(tp,size)]
    else:
        typestr = f"cpptype[{descriptor.cpp_type_name}]"
    return typestr

def string_describe_ft_variant(descriptor: variant_descriptor):
    if descriptor.variant_type == SIMPLEVAR_TYPE:
        return _string_describe_simple_variant(descriptor)
    elif descriptor.variant_type == ENUM_TYPE:
        enum_name = descriptor.enum_desc.enum_name.replace("::", "_")
        return "enum[%s]" % enum_name
    elif descriptor.variant_type == STRUCT_TYPE:
        struct_name = descriptor.struct_desc.struct_name.replace("::", "_")
        return "struct[%s]" % struct_name
    elif descriptor.variant_type == TUPLE_TYPE:
        types = ",".join(string_describe_ft_variant(_) for _ in descriptor.tuple_desc.tuple_map)
        return "tuple[%s]" % types
    elif descriptor.variant_type == UNION_TYPE:
        types = "/".join(string_describe_ft_variant(_) for _ in descriptor.union_desc.possible_types)
        return "union[%s]" % types
    elif descriptor.variant_type == HOMOG_ARRAY_TYPE:
        descstr = string_describe_ft_variant(descriptor.homog_array_desc.value_type)
        return "array[%s]" % descstr
    else:
        warnings.warn("encountered unknown variant type when trying to describe")
        return f"cpptype[{descriptor.cpp_type_name}]"




# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: ft_auth.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import ft_error_pb2 as ft__error__pb2
import ft_types_pb2 as ft__types__pb2
from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\rft_auth.proto\x12\x07\x66oxtrot\x1a\x0e\x66t_error.proto\x1a\x0e\x66t_types.proto\x1a\x1fgoogle/protobuf/timestamp.proto\"|\n\tauth_data\x12\x0e\n\x06userid\x18\x01 \x01(\t\x12\r\n\x05token\x18\x02 \x01(\x0c\x12\x11\n\tauthlevel\x18\x03 \x01(\x05\x12\x11\n\tauthflags\x18\x04 \x01(\x05\x12*\n\x06\x65xpiry\x18\x05 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\"\x1e\n\x0c\x61uth_request\x12\x0e\n\x06userid\x18\x01 \x01(\t\"Y\n\x0e\x61uth_challenge\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\x13\n\x0b\x63hallengeid\x18\x02 \x01(\r\x12\x11\n\tchallenge\x18\x03 \x01(\x0c\"A\n\rauth_response\x12\x13\n\x0b\x63hallengeid\x18\x01 \x01(\r\x12\x0e\n\x06userid\x18\x02 \x01(\t\x12\x0b\n\x03sig\x18\x03 \x01(\x0c\"f\n\x0c\x61uth_confirm\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\x12\n\nsessionkey\x18\x02 \x01(\x0c\x12\x0e\n\x06\x65xpiry\x18\x03 \x01(\x04\x12\x11\n\tauthlevel\x18\x04 \x01(\x05\"i\n\x0esasl_auth_data\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\x11\n\tsasl_data\x18\x02 \x01(\x0c\x12#\n\x07\x61uthdat\x18\x03 \x01(\x0b\x32\x12.foxtrot.auth_data\"I\n\x0e\x61uth_type_list\x12\x1f\n\x03\x65rr\x18\x01 \x01(\x0b\x32\x12.foxtrot.errstatus\x12\x16\n\x0emechanism_name\x18\x02 \x03(\t2\xe6\x02\n\x04\x61uth\x12\x46\n\x14RequestAuthChallenge\x12\x15.foxtrot.auth_request\x1a\x17.foxtrot.auth_challenge\x12\x45\n\x14RespondAuthChallenge\x12\x16.foxtrot.auth_response\x1a\x15.foxtrot.auth_confirm\x12\x45\n\x1aGetSupportedAuthMechanisms\x12\x0e.foxtrot.empty\x1a\x17.foxtrot.auth_type_list\x12G\n\x0fSaslAuthProcess\x12\x17.foxtrot.sasl_auth_data\x1a\x17.foxtrot.sasl_auth_data(\x01\x30\x01\x12?\n\x0bVerifyToken\x12\x17.foxtrot.sasl_auth_data\x1a\x17.foxtrot.sasl_auth_datab\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'ft_auth_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_AUTH_DATA']._serialized_start=91
  _globals['_AUTH_DATA']._serialized_end=215
  _globals['_AUTH_REQUEST']._serialized_start=217
  _globals['_AUTH_REQUEST']._serialized_end=247
  _globals['_AUTH_CHALLENGE']._serialized_start=249
  _globals['_AUTH_CHALLENGE']._serialized_end=338
  _globals['_AUTH_RESPONSE']._serialized_start=340
  _globals['_AUTH_RESPONSE']._serialized_end=405
  _globals['_AUTH_CONFIRM']._serialized_start=407
  _globals['_AUTH_CONFIRM']._serialized_end=509
  _globals['_SASL_AUTH_DATA']._serialized_start=511
  _globals['_SASL_AUTH_DATA']._serialized_end=616
  _globals['_AUTH_TYPE_LIST']._serialized_start=618
  _globals['_AUTH_TYPE_LIST']._serialized_end=691
  _globals['_AUTH']._serialized_start=694
  _globals['_AUTH']._serialized_end=1052
# @@protoc_insertion_point(module_scope)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

import ft_capability_pb2 as ft__capability__pb2
import ft_error_pb2 as ft__error__pb2
import ft_types_pb2 as ft__types__pb2


class capabilityStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.InvokeCapability = channel.unary_unary(
                '/foxtrot.capability/InvokeCapability',
                request_serializer=ft__capability__pb2.capability_request.SerializeToString,
                response_deserializer=ft__capability__pb2.capability_response.FromString,
                )
        self.DescribeServer = channel.unary_unary(
                '/foxtrot.capability/DescribeServer',
                request_serializer=ft__types__pb2.empty.SerializeToString,
                response_deserializer=ft__capability__pb2.servdescribe.FromString,
                )
        self.FetchData = channel.unary_stream(
                '/foxtrot.capability/FetchData',
                request_serializer=ft__capability__pb2.chunk_request.SerializeToString,
                response_deserializer=ft__capability__pb2.datachunk.FromString,
                )
        self.CloseDevice = channel.unary_unary(
                '/foxtrot.capability/CloseDevice',
                request_serializer=ft__capability__pb2.devicechooser.SerializeToString,
                response_deserializer=ft__error__pb2.errstatus.FromString,
                )
        self.RescanDevices = channel.unary_unary(
                '/foxtrot.capability/RescanDevices',
                request_serializer=ft__types__pb2.empty.SerializeToString,
                response_deserializer=ft__error__pb2.errstatus.FromString,
                )
        self.SetupDevice = channel.unary_unary(
                '/foxtrot.capability/SetupDevice',
                request_serializer=ft__capability__pb2.device_setup_request.SerializeToString,
                response_deserializer=ft__capability__pb2.device_setup_response.FromString,
                )
        self.ListHandles = channel.unary_unary(
                '/foxtrot.capability/ListHandles',
                request_serializer=ft__types__pb2.empty.SerializeToString,
                response_deserializer=ft__capability__pb2.handlelist.FromString,
                )
        self.DisposeHandle = channel.unary_unary(
                '/foxtrot.capability/DisposeHandle',
                request_serializer=ft__capability__pb2.handlechooser.SerializeToString,
                response_deserializer=ft__error__pb2.errstatus.FromString,
                )
        self.StreamCapability = channel.stream_stream(
                '/foxtrot.capability/StreamCapability',
                request_serializer=ft__capability__pb2.stream_control.SerializeToString,
                response_deserializer=ft__capability__pb2.capability_response.FromString,
                )
        self.BulkDataDownload = channel.unary_stream(
                '/foxtrot.capability/BulkDataDownload',
                request_serializer=ft__capability__pb2.bulkdata_request.SerializeToString,
                response_deserializer=ft__capability__pb2.bulkdata_chunk.FromString,
                )
        self.InvokeServerAsync = channel.stream_stream(
                '/foxtrot.capability/InvokeServerAsync',
                request_serializer=ft__capability__pb2.async_control.SerializeToString,
                response_deserializer=ft__capability__pb2.async_response.FromString,
                )


class capabilityServicer(object):
    """Missing associated documentation comment in .proto file."""

    def InvokeCapability(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DescribeServer(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def FetchData(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CloseDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def RescanDevices(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def SetupDevice(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListHandles(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DisposeHandle(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamCapability(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def BulkDataDownload(self, request, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def InvokeServerAsync(self, request_iterator, context):
        """Missing associated documentation comment in .proto file."""
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_capabilityServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'InvokeCapability': grpc.unary_unary_rpc_method_handler(
                    servicer.InvokeCapability,
                    request_deserializer=ft__capability__pb2.capability_request.FromString,
                    response_serializer=ft__capability__pb2.capability_response.SerializeToString,
            ),
            'DescribeServer': grpc.unary_unary_rpc_method_handler(
                    servicer.DescribeServer,
                    request_deserializer=ft__types__pb2.empty.FromString,
                    response_serializer=ft__capability__pb2.servdescribe.SerializeToString,
            ),
            'FetchData': grpc.unary_stream_rpc_method_handler(
                    servicer.FetchData,
                    request_deserializer=ft__capability__pb2.chunk_request.FromString,
                    response_serializer=ft__capability__pb2.datachunk.SerializeToString,
            ),
            'CloseDevice': grpc.unary_unary_rpc_method_handler(
                    servicer.CloseDevice,
                    request_deserializer=ft__capability__pb2.devicechooser.FromString,
                    response_serializer=ft__error__pb2.errstatus.SerializeToString,
            ),
            'RescanDevices': grpc.unary_unary_rpc_method_handler(
                    servicer.RescanDevices,
                    request_deserializer=ft__types__pb2.empty.FromString,
                    response_serializer=ft__error__pb2.errstatus.SerializeToString,
            ),
            'SetupDevice': grpc.unary_unary_rpc_method_handler(
                    servicer.SetupDevice,
                    request_deserializer=ft__capability__pb2.device_setup_request.FromString,
                    response_serializer=ft__capability__pb2.device_setup_response.SerializeToString,
            ),
            'ListHandles': grpc.unary_unary_rpc_method_handler(
                    servicer.ListHandles,
                    request_deserializer=ft__types__pb2.empty.FromString,
                    response_serializer=ft__capability__pb2.handlelist.SerializeToString,
            ),
            'DisposeHandle': grpc.unary_unary_rpc_method_handler(
                    servicer.DisposeHandle,
                    request_deserializer=ft__capability__pb2.handlechooser.FromString,
                    response_serializer=ft__error__pb2.errstatus.SerializeToString,
            ),
            'StreamCapability': grpc.stream_stream_rpc_method_handler(
                    servicer.StreamCapability,
                    request_deserializer=ft__capability__pb2.stream_control.FromString,
                    response_serializer=ft__capability__pb2.capability_response.SerializeToString,
            ),
            'BulkDataDownload': grpc.unary_stream_rpc_method_handler(
                    servicer.BulkDataDownload,
                    request_deserializer=ft__capability__pb2.bulkdata_request.FromString,
                    response_serializer=ft__capability__pb2.bulkdata_chunk.SerializeToString,
            ),
            'InvokeServerAsync': grpc.stream_stream_rpc_method_handler(
                    servicer.InvokeServerAsync,
                    request_deserializer=ft__capability__pb2.async_control.FromString,
                    response_serializer=ft__capability__pb2.async_response.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'foxtrot.capability', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class capability(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def InvokeCapability(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.capability/InvokeCapability',
            ft__capability__pb2.capability_request.SerializeToString,
            ft__capability__pb2.capability_response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DescribeServer(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.capability/DescribeServer',
            ft__types__pb2.empty.SerializeToString,
            ft__capability__pb2.servdescribe.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def FetchData(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/foxtrot.capability/FetchData',
            ft__capability__pb2.chunk_request.SerializeToString,
            ft__capability__pb2.datachunk.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CloseDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.capability/CloseDevice',
            ft__capability__pb2.devicechooser.SerializeToString,
            ft__error__pb2.errstatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def RescanDevices(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.capability/RescanDevices',
            ft__types__pb2.empty.SerializeToString,
            ft__error__pb2.errstatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def SetupDevice(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.capability/SetupDevice',
            ft__capability__pb2.device_setup_request.SerializeToString,
            ft__capability__pb2.device_setup_response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListHandles(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.capability/ListHandles',
            ft__types__pb2.empty.SerializeToString,
            ft__capability__pb2.handlelist.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DisposeHandle(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/foxtrot.capability/DisposeHandle',
            ft__capability__pb2.handlechooser.SerializeToString,
            ft__error__pb2.errstatus.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamCapability(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/foxtrot.capability/StreamCapability',
            ft__capability__pb2.stream_control.SerializeToString,
            ft__capability__pb2.capability_response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def BulkDataDownload(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/foxtrot.capability/BulkDataDownload',
            ft__capability__pb2.bulkdata_request.SerializeToString,
            ft__capability__pb2.bulkdata_chunk.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def InvokeServerAsync(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/foxtrot.capability/InvokeServerAsync',
            ft__capability__pb2.async_control.SerializeToString,
            ft__capability__pb2.async_response.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Apr 19 11:44:13 2017

@author: weatherill
"""

import struct
import grpc
from itertools import chain
from .protos.foxtrot_pb2_grpc import exptserveStub
from .protos.ft_capability_pb2_grpc import capabilityStub
from .protos.ft_flags_pb2_grpc import flagsStub
from .common import _check_repl_err
from .server_flags import ServerFlag, FlagProxy
from .TypeConversion import ft_variant_from_value, value_from_ft_variant
from .TypeConversion import string_describe_ft_variant
from .protos.ft_types_pb2 import empty
from .protos.foxtrot_pb2 import  broadcast_notification
from .protos.ft_capability_pb2 import capability_request, capability_argument, VALUE_READONLY, VALUE_READWRITE, ACTION, STREAM
from .protos.ft_types_pb2 import UCHAR_TYPE, USHORT_TYPE, UINT_TYPE, BDOUBLE_TYPE, IINT_TYPE
from .protos.ft_capability_pb2 import chunk_request
from .protos.ft_types_pb2 import ENUM_TYPE
from .EnumCreator import define_enum


DEFAULT_CHUNKSIZE = 1000

class Client:
    def __init__(self, connstr: str, certfile: str = None) -> None:
        if certfile is None:
            self._channel = grpc.insecure_channel(connstr)
        else:
            with open(certfile, "rb") as f:
                cert = f.read()
            creds = grpc.ssl_channel_credentials(root_certificates=cert)
            self._channel = grpc.secure_channel(connstr, creds)
        self._stub = capabilityStub(self._channel)
        self._flagstub = flagsStub(self._channel)
        self._estub = exptserveStub(self._channel)

        self._enum_descs = []
        self._enum_types = []
        self._active_session = None

        self._custom_attr_set = set()

        self.reload_devices()

    def _add_enum_type(self, enumdesc) -> None:
        if enumdesc not in self._enum_descs:
            self._enum_descs.append(enumdesc)
            self._enum_types.append(define_enum(enumdesc))

    def _lookup_enum_type(self, enumdesc):
        if enumdesc in self._enum_descs:
            idx = self._enum_descs.index(enumdesc)
            return self._enum_types[idx]
        return None

    def _setup_device_tree(self) -> None:
        self._devices = []

        for devkey in self._servdescribe.devs_attached.keys():
            dev = self._servdescribe.devs_attached.get(devkey)
            self._devices.append(Device(dev.devid, dev.devtype, dev.caps,
                                        dev.devcomment, self))


        for dev in self._devices:
            if dev._comment:
                setattr(self, dev._comment, dev)
                self._custom_attr_set.add(dev._comment)
            else:
                setattr(self, dev._devtp, dev)
                self._custom_attr_set.add(dev._devtp)

    def reload_devices(self):
        for attrname in self._custom_attr_set:
            delattr(self, attrname)
        self._custom_attr_set.clear()
        
        self._servdescribe = _check_repl_err(self._stub.DescribeServer(empty()))
        self._comment = self._servdescribe.servcomment
        self._setup_device_tree()

    def save_servdesc(self, fname):
        ss = self._servdescribe.SerializeToString()
        with open(fname, 'wb') as f:
            f.write(ss)

    def __getitem__(self, keystr):
        if not hasattr(self, "_devtypes"):
            self._devtypes = [_._devtp for _ in self._devices]
            self._devcomments = [_._comment for _ in self._devices]
            self._devids = [_._devid for _ in self._devices]

        if isinstance(keystr, int):
            return self._devices[self._devids.index(keystr)]

        if keystr in self._devcomments:
            return self._devices[self._devcomments.index(keystr)]

        return self._devices[self._devtypes.index(keystr)]

    def get_all_flags_dict(self) -> dict:
        return {_.name : _ for _ in self.flags}

    def get_all_flags_list(self) -> list:
        return [_ for _ in self.flags]


    def create_new_flag(self, name: str, initval):
        sf = ServerFlag(self, name)
        sf.value = initval
        return sf

    def drop_server_flag(self, flag) -> None:
        if isinstance(flag, str):
            ServerFlag(self, flag).drop()
        elif isinstance(flag, ServerFlag):
            flag.drop()

    def broadcast_notification(self, body: str, title: str = None, channel: str = None):
        req = broadcast_notification(body=body)
        if title is None:
            req.use_default_title = True
            req.title = ""
        else:
            req.title = title

        if channel is None:
            req.use_default_channel = True
            req.channel_target = ""
        else:
            req.channel_target = channel

        repl = self._estub.BroadcastNotification(req)
        _check_repl_err(repl)

    @property
    def flags(self):
        return FlagProxy(self)



class Device:
    def __init__(self, devid, devtp, caps, comment, client):
        self._devid = devid
        self._devtp = devtp
        self._comment = comment
        self._cl = client

        self._caps = []
        self._props = {}
        for cap in caps:
            self._caps.append(Capability(cap.tp, cap.capname, cap.argnames,
                                         cap.argtypes, cap.rettp, devid, client))

        for cap in self._caps:
            setattr(self, cap._capname, cap)
            if(cap._captp == VALUE_READONLY or cap._captp == VALUE_READWRITE):
                propname = "value_%s" % cap._capname
                fget = lambda s: getattr(s,propname)()
                fset = lambda s, val: getattr(s,propname)(val)
                self._props[propname] = property(fget, fset)

    def __repr__(self):
        if self._comment:
            return str("Device(" + self._devtp + ": " + self._comment + ")")

        return str("Device(" + self._devtp + ")")



class Capability:
    def __init__(self, captp, capname, argnames, argtypes, rettp, devid, client):
        self._captp = captp
        self._capname = capname
        self._argnames = argnames
        self._argtypes = argtypes
        self._rettp = rettp
        self._msgid = 0
        self.chunksize = DEFAULT_CHUNKSIZE
        self._devid = devid
        self._cl = client
        self._enum_return_type = None
        self._enum_arg_types = [None] * len(argtypes)

        for tp in chain([rettp], argtypes):
            if tp.variant_type == ENUM_TYPE:
                client._add_enum_type(tp.enum_desc)

        if rettp.variant_type == ENUM_TYPE:
            self._enum_return_type = client._lookup_enum_type(rettp.enum_desc)

        for idx,var in enumerate(argtypes):
            if var.variant_type == ENUM_TYPE:
                self._enum_arg_types[idx] = client._lookup_enum_type(var.enum_desc)

    def __repr__(self):
        if self._captp == VALUE_READONLY:
            infostr = "readonly value"
        elif self._captp == VALUE_READWRITE:
            infostr = "read/write value"
        elif self._captp == ACTION:
            infostr = "action"
        else:
            infostr = "data stream"

        argnamestrs = map(lambda s : "unknown" if not s else s , self._argnames)
        argtypestrs = [string_describe_ft_variant(_) for _ in self._argtypes]
        rettypestr = string_describe_ft_variant(self._rettp)

        argnametypestrs = ["%s:%s" % (n,t) for n,t in zip(argnamestrs,argtypestrs)]

        displaystr = "%s (%s) -> %s, [%s]" % (self._capname, 
                      ", ".join(argnametypestrs),
                      rettypestr,
                      infostr)


        return displaystr

    def get_enum_type(self, argpos):
        if isinstance(argpos, int):
            return self._enum_arg_types[argpos]
        elif isinstance(argpos, str):
            idx = self._argnames.index(argpos)
            return self._enum_arg_types[idx]

    def get_enum(self, argpos, *args, **kwargs):
        tp = self.get_enum_type(argpos)
        return tp(*args, **kwargs)

    def _construct_args(self, *args, **kwargs):
        rawargs = [None] * len(self._argnames)
        for name, val in kwargs.items():
            if name not in self._argnames:
                raise ValueError("no such argument with name: %s in capability"
                                 % name)
            pos = self._argnames.index(name)
            argdesc = self._argtypes[pos]
            rawargs[pos] = capability_argument(
                pos=pos,
                value=ft_variant_from_value(val, argdesc))

        if (len(args) + len(kwargs)) > len(rawargs):
            raise ValueError("too many arguments provided")

        for idx, val in enumerate(args):
            if rawargs[idx] is not None:
                raise IndexError("conflicting positional and keyword arguments")
            desc = self._argtypes[idx]
            rawargs[idx] = capability_argument(position=idx,
                                               value=ft_variant_from_value(val, desc))

        if any(_ is None for _ in rawargs):
            raise IndexError("not all arguments filled in")

        return rawargs

    def _process_sync_response(self, repl):
        if self._captp == STREAM:
            rawbytes = bytearray()

            for chunk in repl:
                _check_repl_err(chunk)
                dtp = chunk.dtp
                rawbytes += chunk.data

            if dtp == UCHAR_TYPE:
                return list(rawbytes)
            if dtp == USHORT_TYPE:
                structstr = '<%dH' % (len(rawbytes) // 2)
            elif dtp == UINT_TYPE:
                structstr = '<%dI' % (len(rawbytes) // 4)
            elif dtp == IINT_TYPE:
                structstr = '<%di' % (len(rawbytes) // 4)
            elif dtp == BDOUBLE_TYPE:
                structstr = '<%dd' % (len(rawbytes) // 8)
            else:
                raise NotImplementedError("data type not supported yet")
            return list(struct.unpack(structstr, rawbytes))

        _check_repl_err(repl)
        out = value_from_ft_variant(repl.returnval)
        return out

    def construct_request(self, *args, **kwargs):
        if self._captp != VALUE_READWRITE:
            capargs = self._construct_args(*args, **kwargs)
        elif len(args) > 0 or len(kwargs) > 0:
            capargs = self._construct_args(*args, **kwargs)
        else:
            capargs = []

        reqtp = chunk_request if self._captp == STREAM else capability_request

        req = reqtp(msgid=self._msgid, devid=self._devid,
                    capname=self._capname, args=capargs)
        self._msgid += 1

        if self._captp == STREAM:
            req.chunksize = self.chunksize

        return req

    def call_cap_sync(self, client, *args, **kwargs):
        stubfun = client._stub.FetchData if self._captp == STREAM else client._stub.InvokeCapability
        req = self.construct_request(*args, **kwargs)
        if client._active_session is None:
            ret = stubfun(req)
        else:
            print("session active using metadata")
            metadata = [("session_secret-bin", client._active_session._secret)]
            ret,status = stubfun.with_call(request=req, metadata=metadata)
        return ret

    def __call__(self, *args, **kwargs):
        repl = self.call_cap_sync(self._cl, *args, **kwargs)
        return self._process_sync_response(repl)



import ft_error_pb2 as _ft_error_pb2
import ft_types_pb2 as _ft_types_pb2
from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class auth_data(_message.Message):
    __slots__ = ["userid", "token", "authlevel", "authflags", "expiry"]
    USERID_FIELD_NUMBER: _ClassVar[int]
    TOKEN_FIELD_NUMBER: _ClassVar[int]
    AUTHLEVEL_FIELD_NUMBER: _ClassVar[int]
    AUTHFLAGS_FIELD_NUMBER: _ClassVar[int]
    EXPIRY_FIELD_NUMBER: _ClassVar[int]
    userid: str
    token: bytes
    authlevel: int
    authflags: int
    expiry: _timestamp_pb2.Timestamp
    def __init__(self, userid: _Optional[str] = ..., token: _Optional[bytes] = ..., authlevel: _Optional[int] = ..., authflags: _Optional[int] = ..., expiry: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ...) -> None: ...

class auth_request(_message.Message):
    __slots__ = ["userid"]
    USERID_FIELD_NUMBER: _ClassVar[int]
    userid: str
    def __init__(self, userid: _Optional[str] = ...) -> None: ...

class auth_challenge(_message.Message):
    __slots__ = ["err", "challengeid", "challenge"]
    ERR_FIELD_NUMBER: _ClassVar[int]
    CHALLENGEID_FIELD_NUMBER: _ClassVar[int]
    CHALLENGE_FIELD_NUMBER: _ClassVar[int]
    err: _ft_error_pb2.errstatus
    challengeid: int
    challenge: bytes
    def __init__(self, err: _Optional[_Union[_ft_error_pb2.errstatus, _Mapping]] = ..., challengeid: _Optional[int] = ..., challenge: _Optional[bytes] = ...) -> None: ...

class auth_response(_message.Message):
    __slots__ = ["challengeid", "userid", "sig"]
    CHALLENGEID_FIELD_NUMBER: _ClassVar[int]
    USERID_FIELD_NUMBER: _ClassVar[int]
    SIG_FIELD_NUMBER: _ClassVar[int]
    challengeid: int
    userid: str
    sig: bytes
    def __init__(self, challengeid: _Optional[int] = ..., userid: _Optional[str] = ..., sig: _Optional[bytes] = ...) -> None: ...

class auth_confirm(_message.Message):
    __slots__ = ["err", "sessionkey", "expiry", "authlevel"]
    ERR_FIELD_NUMBER: _ClassVar[int]
    SESSIONKEY_FIELD_NUMBER: _ClassVar[int]
    EXPIRY_FIELD_NUMBER: _ClassVar[int]
    AUTHLEVEL_FIELD_NUMBER: _ClassVar[int]
    err: _ft_error_pb2.errstatus
    sessionkey: bytes
    expiry: int
    authlevel: int
    def __init__(self, err: _Optional[_Union[_ft_error_pb2.errstatus, _Mapping]] = ..., sessionkey: _Optional[bytes] = ..., expiry: _Optional[int] = ..., authlevel: _Optional[int] = ...) -> None: ...

class sasl_auth_data(_message.Message):
    __slots__ = ["err", "sasl_data", "authdat"]
    ERR_FIELD_NUMBER: _ClassVar[int]
    SASL_DATA_FIELD_NUMBER: _ClassVar[int]
    AUTHDAT_FIELD_NUMBER: _ClassVar[int]
    err: _ft_error_pb2.errstatus
    sasl_data: bytes
    authdat: auth_data
    def __init__(self, err: _Optional[_Union[_ft_error_pb2.errstatus, _Mapping]] = ..., sasl_data: _Optional[bytes] = ..., authdat: _Optional[_Union[auth_data, _Mapping]] = ...) -> None: ...

class auth_type_list(_message.Message):
    __slots__ = ["err", "mechanism_name"]
    ERR_FIELD_NUMBER: _ClassVar[int]
    MECHANISM_NAME_FIELD_NUMBER: _ClassVar[int]
    err: _ft_error_pb2.errstatus
    mechanism_name: _containers.RepeatedScalarFieldContainer[str]
    def __init__(self, err: _Optional[_Union[_ft_error_pb2.errstatus, _Mapping]] = ..., mechanism_name: _Optional[_Iterable[str]] = ...) -> None: ...



from google.protobuf import timestamp_pb2 as _timestamp_pb2
from google.protobuf.internal import containers as _containers
from google.protobuf.internal import enum_type_wrapper as _enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from typing import ClassVar as _ClassVar, Iterable as _Iterable, Mapping as _Mapping, Optional as _Optional, Union as _Union

DESCRIPTOR: _descriptor.FileDescriptor

class simplevalue_types(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    FLOAT_TYPE: _ClassVar[simplevalue_types]
    INT_TYPE: _ClassVar[simplevalue_types]
    UNSIGNED_TYPE: _ClassVar[simplevalue_types]
    BOOL_TYPE: _ClassVar[simplevalue_types]
    STRING_TYPE: _ClassVar[simplevalue_types]
    VOID_TYPE: _ClassVar[simplevalue_types]
    DATETIME_TYPE: _ClassVar[simplevalue_types]
    REMOTE_HANDLE_TYPE: _ClassVar[simplevalue_types]

class byte_data_types(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    UCHAR_TYPE: _ClassVar[byte_data_types]
    CHAR_TYPE: _ClassVar[byte_data_types]
    USHORT_TYPE: _ClassVar[byte_data_types]
    UINT_TYPE: _ClassVar[byte_data_types]
    ULONG_TYPE: _ClassVar[byte_data_types]
    SHORT_TYPE: _ClassVar[byte_data_types]
    IINT_TYPE: _ClassVar[byte_data_types]
    LONG_TYPE: _ClassVar[byte_data_types]
    BFLOAT_TYPE: _ClassVar[byte_data_types]
    BDOUBLE_TYPE: _ClassVar[byte_data_types]
    BCOMPLEX_TYPE: _ClassVar[byte_data_types]
    BDATETIME_TYPE: _ClassVar[byte_data_types]
    BOPAQUE_TYPE: _ClassVar[byte_data_types]

class variant_types(int, metaclass=_enum_type_wrapper.EnumTypeWrapper):
    __slots__ = []
    SIMPLEVAR_TYPE: _ClassVar[variant_types]
    STRUCT_TYPE: _ClassVar[variant_types]
    ENUM_TYPE: _ClassVar[variant_types]
    TUPLE_TYPE: _ClassVar[variant_types]
    HOMOG_ARRAY_TYPE: _ClassVar[variant_types]
    UNION_TYPE: _ClassVar[variant_types]
    DYNAMIC_TYPE: _ClassVar[variant_types]
FLOAT_TYPE: simplevalue_types
INT_TYPE: simplevalue_types
UNSIGNED_TYPE: simplevalue_types
BOOL_TYPE: simplevalue_types
STRING_TYPE: simplevalue_types
VOID_TYPE: simplevalue_types
DATETIME_TYPE: simplevalue_types
REMOTE_HANDLE_TYPE: simplevalue_types
UCHAR_TYPE: byte_data_types
CHAR_TYPE: byte_data_types
USHORT_TYPE: byte_data_types
UINT_TYPE: byte_data_types
ULONG_TYPE: byte_data_types
SHORT_TYPE: byte_data_types
IINT_TYPE: byte_data_types
LONG_TYPE: byte_data_types
BFLOAT_TYPE: byte_data_types
BDOUBLE_TYPE: byte_data_types
BCOMPLEX_TYPE: byte_data_types
BDATETIME_TYPE: byte_data_types
BOPAQUE_TYPE: byte_data_types
SIMPLEVAR_TYPE: variant_types
STRUCT_TYPE: variant_types
ENUM_TYPE: variant_types
TUPLE_TYPE: variant_types
HOMOG_ARRAY_TYPE: variant_types
UNION_TYPE: variant_types
DYNAMIC_TYPE: variant_types

class empty(_message.Message):
    __slots__ = []
    def __init__(self) -> None: ...

class variant_descriptor(_message.Message):
    __slots__ = ["variant_type", "simplevalue_type", "simplevalue_sizeof", "struct_desc", "enum_desc", "tuple_desc", "homog_array_desc", "union_desc", "cpp_type_name", "is_nullable"]
    VARIANT_TYPE_FIELD_NUMBER: _ClassVar[int]
    SIMPLEVALUE_TYPE_FIELD_NUMBER: _ClassVar[int]
    SIMPLEVALUE_SIZEOF_FIELD_NUMBER: _ClassVar[int]
    STRUCT_DESC_FIELD_NUMBER: _ClassVar[int]
    ENUM_DESC_FIELD_NUMBER: _ClassVar[int]
    TUPLE_DESC_FIELD_NUMBER: _ClassVar[int]
    HOMOG_ARRAY_DESC_FIELD_NUMBER: _ClassVar[int]
    UNION_DESC_FIELD_NUMBER: _ClassVar[int]
    CPP_TYPE_NAME_FIELD_NUMBER: _ClassVar[int]
    IS_NULLABLE_FIELD_NUMBER: _ClassVar[int]
    variant_type: variant_types
    simplevalue_type: simplevalue_types
    simplevalue_sizeof: int
    struct_desc: struct_descriptor
    enum_desc: enum_descriptor
    tuple_desc: tuple_descriptor
    homog_array_desc: homog_array_descriptor
    union_desc: union_descriptor
    cpp_type_name: str
    is_nullable: bool
    def __init__(self, variant_type: _Optional[_Union[variant_types, str]] = ..., simplevalue_type: _Optional[_Union[simplevalue_types, str]] = ..., simplevalue_sizeof: _Optional[int] = ..., struct_desc: _Optional[_Union[struct_descriptor, _Mapping]] = ..., enum_desc: _Optional[_Union[enum_descriptor, _Mapping]] = ..., tuple_desc: _Optional[_Union[tuple_descriptor, _Mapping]] = ..., homog_array_desc: _Optional[_Union[homog_array_descriptor, _Mapping]] = ..., union_desc: _Optional[_Union[union_descriptor, _Mapping]] = ..., cpp_type_name: _Optional[str] = ..., is_nullable: bool = ...) -> None: ...

class struct_descriptor(_message.Message):
    __slots__ = ["struct_name", "struct_map"]
    class StructMapEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: variant_descriptor
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[variant_descriptor, _Mapping]] = ...) -> None: ...
    STRUCT_NAME_FIELD_NUMBER: _ClassVar[int]
    STRUCT_MAP_FIELD_NUMBER: _ClassVar[int]
    struct_name: str
    struct_map: _containers.MessageMap[str, variant_descriptor]
    def __init__(self, struct_name: _Optional[str] = ..., struct_map: _Optional[_Mapping[str, variant_descriptor]] = ...) -> None: ...

class enum_descriptor(_message.Message):
    __slots__ = ["enum_name", "enum_map"]
    class EnumMapEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: int
        def __init__(self, key: _Optional[str] = ..., value: _Optional[int] = ...) -> None: ...
    ENUM_NAME_FIELD_NUMBER: _ClassVar[int]
    ENUM_MAP_FIELD_NUMBER: _ClassVar[int]
    enum_name: str
    enum_map: _containers.ScalarMap[str, int]
    def __init__(self, enum_name: _Optional[str] = ..., enum_map: _Optional[_Mapping[str, int]] = ...) -> None: ...

class tuple_descriptor(_message.Message):
    __slots__ = ["tuple_map"]
    TUPLE_MAP_FIELD_NUMBER: _ClassVar[int]
    tuple_map: _containers.RepeatedCompositeFieldContainer[variant_descriptor]
    def __init__(self, tuple_map: _Optional[_Iterable[_Union[variant_descriptor, _Mapping]]] = ...) -> None: ...

class homog_array_descriptor(_message.Message):
    __slots__ = ["has_fixed_size", "fixed_size", "value_type"]
    HAS_FIXED_SIZE_FIELD_NUMBER: _ClassVar[int]
    FIXED_SIZE_FIELD_NUMBER: _ClassVar[int]
    VALUE_TYPE_FIELD_NUMBER: _ClassVar[int]
    has_fixed_size: bool
    fixed_size: int
    value_type: variant_descriptor
    def __init__(self, has_fixed_size: bool = ..., fixed_size: _Optional[int] = ..., value_type: _Optional[_Union[variant_descriptor, _Mapping]] = ...) -> None: ...

class union_descriptor(_message.Message):
    __slots__ = ["possible_types"]
    POSSIBLE_TYPES_FIELD_NUMBER: _ClassVar[int]
    possible_types: _containers.RepeatedCompositeFieldContainer[variant_descriptor]
    def __init__(self, possible_types: _Optional[_Iterable[_Union[variant_descriptor, _Mapping]]] = ...) -> None: ...

class ft_variant(_message.Message):
    __slots__ = ["simplevar", "structval", "enumval", "tupleval", "arrayval"]
    SIMPLEVAR_FIELD_NUMBER: _ClassVar[int]
    STRUCTVAL_FIELD_NUMBER: _ClassVar[int]
    ENUMVAL_FIELD_NUMBER: _ClassVar[int]
    TUPLEVAL_FIELD_NUMBER: _ClassVar[int]
    ARRAYVAL_FIELD_NUMBER: _ClassVar[int]
    simplevar: ft_simplevariant
    structval: ft_struct
    enumval: ft_enum
    tupleval: ft_tuple
    arrayval: ft_homog_array
    def __init__(self, simplevar: _Optional[_Union[ft_simplevariant, _Mapping]] = ..., structval: _Optional[_Union[ft_struct, _Mapping]] = ..., enumval: _Optional[_Union[ft_enum, _Mapping]] = ..., tupleval: _Optional[_Union[ft_tuple, _Mapping]] = ..., arrayval: _Optional[_Union[ft_homog_array, _Mapping]] = ...) -> None: ...

class ft_simplevariant(_message.Message):
    __slots__ = ["dblval", "intval", "uintval", "boolval", "stringval", "tstampval", "handleval", "size", "is_void"]
    DBLVAL_FIELD_NUMBER: _ClassVar[int]
    INTVAL_FIELD_NUMBER: _ClassVar[int]
    UINTVAL_FIELD_NUMBER: _ClassVar[int]
    BOOLVAL_FIELD_NUMBER: _ClassVar[int]
    STRINGVAL_FIELD_NUMBER: _ClassVar[int]
    TSTAMPVAL_FIELD_NUMBER: _ClassVar[int]
    HANDLEVAL_FIELD_NUMBER: _ClassVar[int]
    SIZE_FIELD_NUMBER: _ClassVar[int]
    IS_VOID_FIELD_NUMBER: _ClassVar[int]
    dblval: float
    intval: int
    uintval: int
    boolval: bool
    stringval: str
    tstampval: _timestamp_pb2.Timestamp
    handleval: int
    size: int
    is_void: bool
    def __init__(self, dblval: _Optional[float] = ..., intval: _Optional[int] = ..., uintval: _Optional[int] = ..., boolval: bool = ..., stringval: _Optional[str] = ..., tstampval: _Optional[_Union[_timestamp_pb2.Timestamp, _Mapping]] = ..., handleval: _Optional[int] = ..., size: _Optional[int] = ..., is_void: bool = ...) -> None: ...

class ft_struct(_message.Message):
    __slots__ = ["struct_name", "value"]
    class ValueEntry(_message.Message):
        __slots__ = ["key", "value"]
        KEY_FIELD_NUMBER: _ClassVar[int]
        VALUE_FIELD_NUMBER: _ClassVar[int]
        key: str
        value: ft_variant
        def __init__(self, key: _Optional[str] = ..., value: _Optional[_Union[ft_variant, _Mapping]] = ...) -> None: ...
    STRUCT_NAME_FIELD_NUMBER: _ClassVar[int]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    struct_name: str
    value: _containers.MessageMap[str, ft_variant]
    def __init__(self, struct_name: _Optional[str] = ..., value: _Optional[_Mapping[str, ft_variant]] = ...) -> None: ...

class ft_enum(_message.Message):
    __slots__ = ["desc", "enum_value"]
    DESC_FIELD_NUMBER: _ClassVar[int]
    ENUM_VALUE_FIELD_NUMBER: _ClassVar[int]
    desc: enum_descriptor
    enum_value: int
    def __init__(self, desc: _Optional[_Union[enum_descriptor, _Mapping]] = ..., enum_value: _Optional[int] = ...) -> None: ...

class ft_tuple(_message.Message):
    __slots__ = ["value"]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    value: _containers.RepeatedCompositeFieldContainer[ft_variant]
    def __init__(self, value: _Optional[_Iterable[_Union[ft_variant, _Mapping]]] = ...) -> None: ...

class ft_homog_array(_message.Message):
    __slots__ = ["arr_decoded", "arr_encoded", "arr_heavy"]
    ARR_DECODED_FIELD_NUMBER: _ClassVar[int]
    ARR_ENCODED_FIELD_NUMBER: _ClassVar[int]
    ARR_HEAVY_FIELD_NUMBER: _ClassVar[int]
    arr_decoded: ft_homog_array_decoded
    arr_encoded: ft_homog_array_encoded
    arr_heavy: ft_heavy_array
    def __init__(self, arr_decoded: _Optional[_Union[ft_homog_array_decoded, _Mapping]] = ..., arr_encoded: _Optional[_Union[ft_homog_array_encoded, _Mapping]] = ..., arr_heavy: _Optional[_Union[ft_heavy_array, _Mapping]] = ...) -> None: ...

class ft_homog_array_encoded(_message.Message):
    __slots__ = ["dtp", "data"]
    DTP_FIELD_NUMBER: _ClassVar[int]
    DATA_FIELD_NUMBER: _ClassVar[int]
    dtp: byte_data_types
    data: bytes
    def __init__(self, dtp: _Optional[_Union[byte_data_types, str]] = ..., data: _Optional[bytes] = ...) -> None: ...

class ft_dynamic_type(_message.Message):
    __slots__ = ["value", "desc"]
    VALUE_FIELD_NUMBER: _ClassVar[int]
    DESC_FIELD_NUMBER: _ClassVar[int]
    value: ft_variant
    desc: variant_descriptor
    def __init__(self, value: _Optional[_Union[ft_variant, _Mapping]] = ..., desc: _Optional[_Union[variant_descriptor, _Mapping]] = ...) -> None: ...

class ft_heavy_array(_message.Message):
    __slots__ = ["data"]
    DATA_FIELD_NUMBER: _ClassVar[int]
    data: _containers.RepeatedCompositeFieldContainer[ft_variant]
    def __init__(self, data: _Optional[_Iterable[_Union[ft_variant, _Mapping]]] = ...) -> None: ...

class ft_homog_array_decoded(_message.Message):
    __slots__ = ["arr_uint8", "arr_uint16", "arr_sint16", "arr_uint32", "arr_sint32", "arr_uint64", "arr_sint64", "arr_float", "arr_double"]
    ARR_UINT8_FIELD_NUMBER: _ClassVar[int]
    ARR_UINT16_FIELD_NUMBER: _ClassVar[int]
    ARR_SINT16_FIELD_NUMBER: _ClassVar[int]
    ARR_UINT32_FIELD_NUMBER: _ClassVar[int]
    ARR_SINT32_FIELD_NUMBER: _ClassVar[int]
    ARR_UINT64_FIELD_NUMBER: _ClassVar[int]
    ARR_SINT64_FIELD_NUMBER: _ClassVar[int]
    ARR_FLOAT_FIELD_NUMBER: _ClassVar[int]
    ARR_DOUBLE_FIELD_NUMBER: _ClassVar[int]
    arr_uint8: ft_uint8_arr
    arr_uint16: ft_uint32_arr
    arr_sint16: ft_sint32_arr
    arr_uint32: ft_uint32_arr
    arr_sint32: ft_sint32_arr
    arr_uint64: ft_uint64_arr
    arr_sint64: ft_sint64_arr
    arr_float: ft_float_arr
    arr_double: ft_double_arr
    def __init__(self, arr_uint8: _Optional[_Union[ft_uint8_arr, _Mapping]] = ..., arr_uint16: _Optional[_Union[ft_uint32_arr, _Mapping]] = ..., arr_sint16: _Optional[_Union[ft_sint32_arr, _Mapping]] = ..., arr_uint32: _Optional[_Union[ft_uint32_arr, _Mapping]] = ..., arr_sint32: _Optional[_Union[ft_sint32_arr, _Mapping]] = ..., arr_uint64: _Optional[_Union[ft_uint64_arr, _Mapping]] = ..., arr_sint64: _Optional[_Union[ft_sint64_arr, _Mapping]] = ..., arr_float: _Optional[_Union[ft_float_arr, _Mapping]] = ..., arr_double: _Optional[_Union[ft_double_arr, _Mapping]] = ...) -> None: ...

class ft_uint8_arr(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, values: _Optional[_Iterable[int]] = ...) -> None: ...

class ft_uint16_arr(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, values: _Optional[_Iterable[int]] = ...) -> None: ...

class ft_sint16_arr(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, values: _Optional[_Iterable[int]] = ...) -> None: ...

class ft_uint32_arr(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, values: _Optional[_Iterable[int]] = ...) -> None: ...

class ft_sint32_arr(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, values: _Optional[_Iterable[int]] = ...) -> None: ...

class ft_uint64_arr(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, values: _Optional[_Iterable[int]] = ...) -> None: ...

class ft_sint64_arr(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedScalarFieldContainer[int]
    def __init__(self, values: _Optional[_Iterable[int]] = ...) -> None: ...

class ft_float_arr(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedScalarFieldContainer[float]
    def __init__(self, values: _Optional[_Iterable[float]] = ...) -> None: ...

class ft_double_arr(_message.Message):
    __slots__ = ["values"]
    VALUES_FIELD_NUMBER: _ClassVar[int]
    values: _containers.RepeatedScalarFieldContainer[float]
    def __init__(self, values: _Optional[_Iterable[float]] = ...) -> None: ...



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Jul 16 15:04:58 2019

@author: weatherill
"""

#from __future__ import annotations
from pyfoxtrot.common import _check_repl_err
from pyfoxtrot.protos.ft_types_pb2 import  empty
from pyfoxtrot.protos.ft_flags_pb2 import serverflag

class ServerFlag:
    def __init__(self, client, flagname: str):
        self._flagname = flagname
        self._client = client

    @staticmethod
    def construct_request(flagname: str, val=0): #-> serverflag:
        req = serverflag()
        req.msgid = 0
        req.flagname = flagname
        if isinstance(val, float):
            req.dblval = val
        elif isinstance(val, bool):
            req.boolval = val
        elif isinstance(val, int):
            req.intval = val
        elif isinstance(val, str):
            req.stringval = val.encode("ASCII")
        else:
            raise TypeError("invalid value type %s for server flag" % type(val))

        return req

    def __repr__(self) -> str:
        tval = self.value
        tempstr = "ServerFlag( %s : %s (%s))"
        return tempstr % (self._flagname, tval, type(tval).__name__)

    @property
    def name(self) -> str:
        return self._flagname

    def drop(self) -> None:
        req = serverflag()
        req.flagname = self._flagname
        repl = self._client._flagstub.DropServerFlag(req)
        _check_repl_err(repl)

    @property
    def value(self):
        req = self.construct_request(self._flagname)
        ret = self._client._flagstub.GetServerFlag(req)
        _check_repl_err(ret)
        whichattr = ret.WhichOneof("arg")
        if whichattr is None:
            return None
        return getattr(ret, whichattr)

    @value.setter
    def value(self, val):
        req = self.construct_request(self._flagname, val)
        repl = self._client._flagstub.SetServerFlag(req)
        _check_repl_err(repl)
        whichattr = repl.WhichOneof("arg")
        if whichattr is None:
            return None
        return getattr(repl, whichattr)

class FlagProxy:
    def __init__(self, cl):
        self._cl = cl

    def __getitem__(self, flagname: str):
        flg = ServerFlag(self._cl, flagname)
        return flg

    def __setitem__(self, flagname: str, val) -> None:
        flg = ServerFlag(self._cl, flagname)
        flg.value = val

    def __iter__(self):
        response = self._cl._flagstub.ListServerFlags(empty())
        _check_repl_err(response)
        self._flaglist = [ServerFlag(self._cl, _.flagname) for _ in response.flags]
        self._flagit = iter(self._flaglist)
        return self

    def __next__(self):
        return next(self._flagit)
